;;;
;;; banyan/camera - cameras
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: camera.scm,v 1.8 2003/01/21 11:19:43 shirok Exp $
;;;

(select-module banyan)
(use math.const)
(use gl.math3d)

;; PROTOTYPE IMPLEMENTATION

(define-class <camera> (<renderable-node>)
  ((aim-point    :init-keyword :aim-point  :accessor aim-point-of
                 :init-form (make-point4f) :attribute-type <bn-vec3>)
   (distance     :init-keyword :distance   :accessor distance-of
                 :init-form 10 :attribute-type <real>)
   (x-rotation   :init-keyword :x-rotation :accessor x-rotation-of
                 :init-form 0  :attribute-type <real>)
   (y-rotation   :init-keyword :y-rotation :accessor y-rotation-of
                 :init-form 0  :attribute-type <real>)
   (fov          :init-keyword :fov        :accessor fov-of
                 :init-form 60.0 :attribute-type <real>)
   (near-plane   :init-keyword :near-plane :accessor near-plane-of
                 :init-form 1.0 :attribute-type <real>)
   (far-plane    :init-keyword :far-plane  :accessor far-plane-of
                 :init-form 1000.0 :attribute-type <real>)
   (cmatrix      :accessor cmatrix-of
                 :init-form (make-matrix4f))
   (perspective  :init-keyword :perspective? :accessor perspective?
                 :init-form #t :attribute-type <boolean>)
   (rotatable    :init-keyword :rotatable? :accessor rotatable?
                 :init-form #t :attribute-type <boolean>)
   ))

(define-method initialize ((self <camera>) initargs)
  (next-method)
  (and-let* ((scene (current-scene)))
    (add-camera! scene self)))

(define-method node-delete ((self <camera>))
  (and-let* ((scene (current-scene)))
    (delete-camera! scene self))
  (next-method))

(define (make-default-camera)
  (with-part (transient-part)
    (make <camera>
      :name "ortho-z"
      :perspective? #f   :rotatable? #f
      :distance 15.0
      :x-rotation 0.0   :y-rotation 0.0)
    (make <camera>
      :name "ortho-y"
      :perspective? #f   :rotatable? #f
      :distance 15.0
      :x-rotation -90.0   :y-rotation 0.0)
    (make <camera>
      :name "ortho-x"
      :perspective? #f   :rotatable? #f
      :distance 15.0
      :x-rotation 0.0    :y-rotation 90.0)
    (make <camera>
      :name "perspcamera"
      :distance 15.0
      :x-rotation -15.0
      :y-rotation 45.0)
    ))

(define camera-update-cmatrix
  (let ((camqx   (make-quatf))
        (camqy   (make-quatf))
        (dis-vec (make-vector4f))
        (aim-mat (make-matrix4f)))
    (lambda (camera)
      (let1 cmatrix (cmatrix-of camera)
        (rotation->quatf! camqx #,(vector4f 1 0 0)
                          (* (- (x-rotation-of camera)) pi/180))
        (rotation->quatf! camqy #,(vector4f 0 1 0)
                          (* (- (y-rotation-of camera)) pi/180))
        (quatf-mul! camqx camqy)
        (vector4f-set! dis-vec 2 (- (distance-of camera)))
        (set! (matrix4f-column aim-mat 3) (aim-point-of camera))
        (tqs->matrix4f! cmatrix dis-vec camqx '#,(vector4f 1 1 1))
        (matrix4f-mul! cmatrix aim-mat)
        cmatrix))
    ))

(provide "banyan/camera")
