;;;
;;; banyan/color - color manipulation
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: color.scm,v 1.3 2003/02/22 14:30:24 shirok Exp $
;;;

(select-module banyan)
(autoload gtk <gdk-color> gdk-colormap-get-system gdk-colormap-alloc-color)

;;========================================================
;; Pseudo types for attribute handling
;; The actual data is stored in some sequence (for now)

(define-class <bn-color-meta> (<class>)
  ())

(define-class <bn-color3> ()
  ()
  :metaclass <bn-color-meta>)

(define-class <bn-color4> ()
  ()
  :metaclass <bn-color-meta>)

;;--------------------------------------------------------
;; Utility routines
;;

(define bn-system-cmap
  (let ((cmap #f))
    (lambda ()
      (unless cmap (set! cmap (gdk-colormap-get-system)))
      cmap)))

(define (bn-color r g b . maybe-a)
  (let ((a (get-optional maybe-a 1.0)))
    (f32vector r g b a)))

(define (bn-color->gdk-color cvec . args)
  (define (scale val)
    (inexact->exact (round (clamp (* val 65535) 0.0 65535.0))))
  (let-optionals* args ((writable? #f)
                        (best-match? #f))
    (let1 color (make <gdk-color>
                  :red   (scale (ref cvec 0))
                  :green (scale (ref cvec 1))
                  :blue  (scale (ref cvec 2)))
      ;; TODO: error check
      (gdk-colormap-alloc-color (bn-system-cmap)
                                color writable? best-match?)
      color)))

(define (gdk-color->bn-color color)
  (define (descale val)
    (clamp (/ val 65535) 0.0 1.0))
  (f32vector (descale (ref color 'red))
             (descale (ref color 'green))
             (descale (ref color 'blue))
             1.0))

                                  