;;;
;;; banyan.gui - GUI for banyan
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: gui.scm,v 1.27 2003/12/17 10:50:09 shirok Exp $
;;;

(define-module banyan.gui
  (extend gauche.mop.singleton gauche.mop.validator gauche.hook
          file.util util.list
          gtk gtk.gtkgl gl
          glim banyan)
  (export dialog-show dialog-show+ dialog-action
          dialog-action-gadget ---- dialog-buttons
          image-button-new
          <banyan-app-browser>
          <banyan-app-composer>)
  )
(select-module banyan.gui)

;; common utilities

(define (dialog-show class)
  (glim-run-frame-top-level (instance-of class)))

(define-macro (dialog-show+ class)
  `(lambda _ (dialog-show ,class)))

(define-generic dialog-action)

(define-macro (dialog-action-gadget label value)
  `(make <glim-push-button> :label ,label
         :activate-callback (lambda _ (dialog-action .frame. ,value))))

(define-macro (----)
  `(glim-gtk-widget->pane (gtk-hseparator-new)))

(define-macro (dialog-buttons . buttons)
  `(glim-horizontally (:homogeneous #t :spacing 3 :thickness 6)
     ,@(map (lambda (x) `(:expand ,x)) buttons)))

(define (image-button-new image-file)
  (let ((img (gtk-image-new-from-file (build-path (image-directory-path) image-file)))
        (button (gtk-button-new)))
    (gtk-container-add button img)
    button))

;; Autoloads dialogs
(autoload banyan.gui.parteditor   part-editor-activate!)
(autoload banyan.gui.attribed     <attribute-editor>
                                  attribute-editor-attach
                                  attribute-editor-detach)
(autoload banyan.gui.shapedialog  <cube-creator-dialog>
                                  <sphere-creator-dialog>
                                  <cylinder-creator-dialog>
                                  <cone-creator-dialog>
                                  <plane-creator-dialog>
                                  <lambda-creator-dialog>
                                  <procedural-creator-dialog>
                                  <fracterrain-creator-dialog>
                                  <lsystem-creator-dialog>)
(autoload banyan.gui.repobrowser  <repository-browser-dialog>)
(autoload banyan.gui.partbrowser  <part-browser>)
(autoload banyan.gui.partdisplay  <part-display>)
(autoload banyan.gui.errordialog  error-dialog)
(autoload banyan.gui.exporter     <exporter-dialog>)
(autoload banyan.gui.nodeselector node-selector-activate!)
(autoload banyan.gui.nodecreator  node-creator-activate!)
(autoload banyan.gui.nodeduplicator <node-duplicate-dialog>)
(autoload banyan.gui.nodepartschanger <node-change-parts-dialog>)
(autoload banyan.gui.transfreezer <transform-freeze-dialog>)
(autoload banyan.gui.shelf        <composer-shelf>)

(autoload banyan.gui.app-browser  <banyan-app-browser>)
(autoload banyan.gui.app-composer <banyan-app-composer>)

;; Each subfile exports its external symbols
(require "banyan/gui/config")
(require "banyan/gui/manipulator")
(require "banyan/gui/mainmenu")
(require "banyan/gui/viewport")
(require "banyan/gui/database")
(require "banyan/gui/partdisplay")

(provide "banyan/gui")
