;;;
;;; banyan.gui.config - global configuration management
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: config.scm,v 1.8 2003/02/23 03:32:54 shirok Exp $
;;;

(select-module banyan.gui)
(use gauche.parameter)
(use gauche.threads)
(use file.util)
(use srfi-2)
(export banyan-init-gui banyan-quit glconfig)

;; PROTOTYPE IMPLEMENTATION

(define glconfig (make-parameter #f))

(define image-directory-path
  (make-parameter
   (if (file-exists? "../images/icon-select.png")
       "../images"
       (build-path (gauche-site-library-directory) "banyan/gui/images"))))


;; Alternative error handler
(define (banyan-report-error exc)
  ;; print stack.  taken from gauche/vm/debugger.
  (define (debug-print-stack stack max-depth)
    (let ((outp (current-error-port)))
      (format outp "Stack trace\n")
      (format outp "________________________________________________\n")
      (do ((s stack (cdr s))
           (i 0     (+ i 1)))
          ((or (null? s) (> i max-depth))
           (unless (null? s) (format outp "...\n")))
        (let ((code (caar s)))
          (format outp "~3d  ~,,,,v:s\n" i 65 code)
          (or (and-let* (((pair? code))
                         (info (pair-attribute-get code 'source-info #f))
                         ((pair? info))
                         ((pair? (cdr info))))
                (format outp "       At line ~a of ~s\n"
                        (cadr info) (car info))
                #t)
              (format outp "       [unknown location]\n"))))
      ))
  (let ((trace (vm-get-stack-trace))
        (msg   (format "*** ERROR: ~a"
                       (if (is-a? exc <error>) (ref exc 'message) exc))))
    (error-dialog "~a" msg)
    (display msg (current-error-port))
    (newline (current-error-port))
    (debug-print-stack (cdr trace) 20))
  )

(define (banyan-init-gui args)
  (gtk-init args)
  (unless (gdk-gl-query-extension)
    (error "OpenGL not supported"))
  (let1 conf (gdk-gl-config-new-by-mode (logior GDK_GL_MODE_RGB
                                                GDK_GL_MODE_DEPTH
                                                GDK_GL_MODE_DOUBLE))
    (unless conf (error "required GL visual not supported"))
    (vm-set-default-exception-handler (current-thread) banyan-report-error)
    (glconfig conf)))

(define (banyan-quit)
  (gtk-main-quit))

(provide "banyan.gui.config")
