;;;
;;; banyan/gui/errordialog.scm - error dialog
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: errordialog.scm,v 1.6 2003/01/14 12:38:57 shirok Exp $
;;;

;; Reports User-operation error.
;; This does not substitute Gauche's report-error handler.  It is
;; reserved to catch an unexpected implementation errors.  The procedure
;; error-dialog should be called when Banyan wants to notify the user
;; some unusual condition.

(define-module banyan.gui.errordialog
  (use gauche.mop.singleton)
  (extend banyan.gui)
  (export error-dialog))
(select-module banyan.gui.errordialog)

;; taken from gtk.error-dialog, customized for banyan.
(define-class <error-dialog> (<singleton-mixin>)
  ((widget)
   (label)
   (parent :init-keyword :parent :init-value #f)
   (flags  :init-keyword :flags  :init-value 0)
   ))

(define-method initialize ((self <error-dialog>) initargs)
  (next-method)
  (let* ((dialog (gtk-dialog-new-with-buttons "Error"
                                              (ref self 'parent)
                                              (ref self 'flags)
                                              GTK_STOCK_OK
                                              GTK_RESPONSE_ACCEPT))
         (vbox   (ref dialog 'vbox))
         (label  (gtk-label-new "")))
    (g-signal-connect dialog "response"
                      (lambda _ (gtk-widget-hide-all dialog)))
    (gtk-box-pack-start vbox label  #t #t 10)
    (slot-set! self 'widget dialog)
    (slot-set! self 'label label)
    ))

(define (error-dialog fmt . args)
  (let ((self (instance-of <error-dialog>)))
    (gtk-label-set-text (ref self 'label) (apply format #f fmt args))
    (gtk-widget-show-all (ref self 'widget))))

(provide "banyan/gui/errordialog")

