;;;
;;; banyan/gui/exporter - scene exporter UI
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: exporter.scm,v 1.2 2003/02/19 12:24:49 shirok Exp $
;;;

(define-module banyan.gui.exporter
  (extend banyan.gui)
  (export <exporter-dialog>))
(select-module banyan.gui.exporter)

(define-glim-application-frame <exporter-dialog> ()
  ()
  (:panes
   (selector    (make <glim-hierarchy-selector-pane>
                  ))
   (export-type (make <glim-option-pane>
                  :name-key cadr :value-key car
                  :items (map (lambda (f)
                                (list (car f) #`",(cadr f) ,(caddr f)"))
                              (list-export-formats))))
   )
  (:layouts
   (default
     (glim-vertically (:thickness 2 :spacing 2)
       (:expand selector)
       (glim-labelling (:label "Export type:" :label-alignment :left
                        :spacing 2 :thickness 2)
         export-type)
       (----)
       (dialog-buttons
        (dialog-action-gadget "OK" 'ok)
        (dialog-action-gadget "Close" 'close))
       )))
  )

;; Action
(define-method dialog-action ((self <exporter-dialog>) type)
  (case type
    ((ok)
     (let* ((export-type (glim-gadget-value
                          (glim-get-frame-pane self 'export-type)))
            (exporter (get-exporter export-type))
            (paths (glim-gadget-value
                    (glim-get-frame-pane self 'selector)))
            (file  (if (null? paths) #f (string-join paths "/" 'prefix)))
            )
       (cond
        ((not file)
         (error-dialog "Select output filename"))
        ((not exporter)
         (error-dialog "Don't know how to export ~a" export-type))
        ((not (file-is-directory? (sys-dirname file)))
         (error-dialog "Directory of ~s doesn't exist" file))
        (else
         (call-with-output-file file
           (lambda (out) (exporter (current-scene) out)))
         (glim-frame-exit self))
        )))
    (else
     (glim-frame-exit self)))
  )

(provide "banyan/gui/exporter")
