;;;
;;; banyan/gui/mainmenu - application main menu
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: mainmenu.scm,v 1.13 2003/02/22 06:30:23 shirok Exp $
;;;

(select-module banyan.gui)
(export main-menu)

;; Returns application's main menu definition (<glim-menu> instance).
;; The menu definition should be realized by glim-popup-menu or
;; glim-pulldown-menu; see lib/glim/menu.scm for details.

(define banyan-composer-main-menu
  (make-parameter
   (glim-menu
    `((menu "Scene" :items
            ((button "New scene" :callback
                     ,(lambda _ (new-scene)))
             (button "Import part ..." :callback
                     ,(dialog-show+ <repository-browser-dialog>))
             (button "Commit all" :callback
                     ,(lambda _ (db-commit-current-scene-to-repository)))
             (button "Export ..." :callback
                     ,(dialog-show+ <exporter-dialog>))
             (separator)
             (button "Quit" :callback ,(lambda _ (banyan-quit)))
             ))
      (menu "Edit" :items
            ((button "Select" :callback
                     ,(lambda _ (manipulator-mode-set! #f)))
             (button "Translaet" :callback
                     ,(lambda _ (manipulator-mode-set! 'translate)))
             (button "Rotate" :callback
                     ,(lambda _ (manipulator-mode-set! 'rotate)))
             (button "Scale" :callback
                     ,(lambda _ (manipulator-mode-set! 'scale)))
             (separator)
             (button "Push selections" :callback
                     ,(lambda _ (push-selections (current-scene))))
             (button "Pop selections" :callback
                     ,(lambda _ (pop-selections (current-scene))))
             (button "Swap selections" :callback
                     ,(lambda _ (swap-selections (current-scene))))
             (button "Clear selection stack" :callback
                     ,(lambda _ (clear-selection-stack (current-scene))))
             (separator)
             (button "Select children" :callback
                     ,(lambda _ (select-children (current-scene))))
             (button "Select parents" :callback
                     ,(lambda _ (select-parents (current-scene))))
             (separator)
             (button "Freeze transform ..." :callback
                     ,(dialog-show+ <transform-freeze-dialog>))
             (button "Parent (child, parent)" :callback
                     ,(lambda _ (parent-selected (current-scene))))
             (button "Unparent" :callback
                     ,(lambda _ (unparent-selected (current-scene))))
             (button "Duplicate ..." :callback
                     ,(dialog-show+ <node-duplicate-dialog>))
             (button "Change Part ..." :callback
                     ,(dialog-show+ <node-change-parts-dialog>))
             (separator)
             (button "Delete" :callback
                     ,(lambda _
                        (delete-instance-selected (current-scene))))
             ))
      (menu "Create" :items
            ((button "Cube" :callback
                     ,(dialog-show+ <cube-creator-dialog>))
             (button "Sphere" :callback
                     ,(dialog-show+ <sphere-creator-dialog>))
             (button "Cylinder" :callback
                     ,(dialog-show+ <cylinder-creator-dialog>))
             (button "Cone" :callback
                     ,(dialog-show+ <cone-creator-dialog>))
             (button "Plane" :callback
                     ,(dialog-show+ <plane-creator-dialog>))
             ;;(button "Teapot" :callback
             ;;        ,(lambda _ (make-teapot)))
             (separator)
             (button "Locator" :callback
                     ,(lambda _ (make-locator)))
             (button "Connector" :callback
                     ,(lambda _
                        (for-each make-connector
                                  (selections-of (current-scene)))))
             (separator)
             (button "Part" :callback
                     ,(lambda _ (db-create-new-part)))
             ))
      (menu "Camera" :items
            ((button "Aim to selected" :callback
                     ,(lambda _ (aim-current-camera (active-viewport))))
             (button "Reset" :callback
                     ,(lambda _ (reset-current-camera (active-viewport))))
             (separator)
             (radio camera/cameras
                    :items () ;;to be added
                    :callback ,(lambda (radio value)
                                 (set-current-camera (active-viewport) value)))
             ))
      (menu "Render" :items
            ((radio :items
                    ((button "Wireframe" :value wire)
                     (button "Wireframe, lighting" :value wirel)
                     (button "Flat shading" :value flat)
                     (button "Smooth shading" :value smooth))
                    :callback
                    ,(lambda (item mode)
                       (set-render-flag (active-viewport) mode)))
             (separator)
             (check render/grid "Grid" :callback
                    ,(lambda (_ checked?)
                       (set-grid-mode (active-viewport) checked?)))
             (check render/normals "Normals" :callback
                    ,(lambda (_ checked?)
                       (shape-set-render-normals!
                        (if checked? 0.5 0))))
             (button "Background color ..." :callback
                     ,(lambda _ (set-background-color-interactively
                                 (active-viewport))))
             ))
      (menu "Window" :items
            ((button "Attribute Editor ..." :callback
                     ,(lambda _
                        (dialog-show <attribute-editor>)
                        (let ((s (selections-of (current-scene))))
                          (unless (null? s)
                            (attribute-editor-attach (car s))))))
             (button "Part Browser ..." :callback
                     ,(dialog-show+ <part-browser>))
             ;(button "Scene Browser ..." :callback
             ;        ,(dialog-show+ <banyan-app-browser>))
             ))
      ))
   ))

(define banyan-browser-main-menu
  (make-parameter
   (glim-menu
    `((menu "Scene" :items
            ((button "New scene" :callback
                     ,(lambda _ (new-scene)))
             (button "Import part ..." :callback
                     ,(dialog-show+ <repository-browser-dialog>))
             (separator)
             (button "Quit" :callback ,(lambda _ (banyan-quit)))
             ))
      (menu "Camera" :items
            ((button "Aim to selected" :callback
                     ,(lambda _ (aim-current-camera (active-viewport))))
             (button "Reset" :callback
                     ,(lambda _ (reset-current-camera (active-viewport))))
             (separator)
             (radio camera/cameras
                    :items () ;;to be added
                    :callback ,(lambda (radio value)
                                 (set-current-camera (active-viewport) value)))
             ))
      (menu "Render" :items
            ((radio :items
                    ((button "Wireframe" :value wire)
                     (button "Wireframe, lighting" :value wirel)
                     (button "Flat shading" :value flat)
                     (button "Smooth shading" :value smooth))
                    :callback
                    ,(lambda (item mode)
                       (set-render-flag (active-viewport) mode)))
             (separator)
             (check render/grid "Grid" :callback
                    ,(lambda (_ checked?)
                       (set-grid-mode (active-viewport) checked?)))
             (check render/normals "Normals" :callback
                    ,(lambda (_ checked?)
                       (shape-set-render-normals!
                        (if checked? 0.5 0))))
             (button "Background color ..." :callback
                     ,(lambda _ (set-background-color-interactively
                                 (active-viewport))))
             ))
      (menu "Window" :items
            ((button "Part Browser ..." :callback
                     ,(dialog-show+ <part-browser>))
             ;(button "Scene Composer ..." :callback
             ;        ,(dialog-show+ <banyan-app-composer>))
             ))
      ))
   ))

;;------------------------------------------------------------------
;; Dynamic menus
;;

(define (main-menu-add-camera! menu camera)
  (and-let* ((cammenu (glim-menu-find-item menu 'camera/cameras)))
    (glim-menu-add-item cammenu `(button ,(name-of camera) :value ,camera))
    ))

(define (main-menu-delete-cameras! menu)
  (and-let* ((cammenu (glim-menu-find-item menu 'camera/cameras)))
    (for-each (cut glim-menu-delete-item cammenu <>)
              (ref cammenu 'items))))

;;------------------------------------------------------------------
;; Auxiliary commands.
;;  I'm not sure whether they should be here or not.

(define-method select-children ((scene <scene>) . maybe-instances)
  (let-optionals* maybe-instances ((instances (selections-of scene)))
    (apply set-selection scene
           (apply lset-union eq? (map children-of instances)))))

(define-method select-parents ((scene <scene>) . maybe-instances)
  (let-optionals* maybe-instances ((instances (selections-of scene)))
    (apply set-selection scene
           (apply lset-union eq? (map parents-of instances)))))

(define-method parent-selected ((scene <scene>))
  (let* ((parent   (selections-of scene))
         (children (peek-selection-stack scene)))
    (cond ((null? parent)
           (error-dialog "select an instance to be a parent"))
          ((not (null? (cdr parent)))
           (error-dialog "select exactly one instance to be a parent"))
          ((null? children)
           (error-dialog "prepare a set of instances to be a child in the selection stack"))
          (else
           (for-each (cute instance-parent <> (car parent)) children)))))

(define-method unparent-selected ((scene <scene>))
  (for-each (cut instance-unparent <>) (selections-of scene)))

(provide "banyan/gui/mainmenu")
