;;;
;;; banyan/gui/nodecreator - generic node creation
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: nodecreator.scm,v 1.5 2003/01/01 07:57:38 shirok Exp $
;;;

(define-module banyan.gui.nodecreator
  (use srfi-1)
  (use srfi-2)
  (use gauche.mop.instance-pool)
  (extend banyan.gui)
  (export node-creator-activate!))
(select-module banyan.gui.nodecreator)

;; A UI to create a node.

;; This dialog provides a generic mechanism to create a node.
;; This is not the only way to create a node---many nodes have
;; specialized way and interface to be created.  This dialog is
;; intended to be called from node selector, i.e. when the user
;; decides she/he needs a connection to the new node.   All attributes
;; are set to default.  The user will adjust them in attribute editor.

;;==================================================================
;; Node creator
;;

(define-glim-application-frame <node-creator-dialog> ()
  ((callback :init-value #f)
   )
  (:panes
   (class-selector (make-class-selector .frame.))
   (name-entry     (glim-gtk-widget->pane (gtk-entry-new)))
   )
  (:layouts
   (default
     (glim-vertically ()
       (make <glim-label-pane> :text "New node")
       (glim-tabling (:spacing 2
                      :column-options '((0 :fill)
                                        ($ :fill :expand)))
         (list (make <glim-label-pane> :text "Class:" :align 'left)
               (glim-get-frame-pane .frame. 'class-selector))
         (list (make <glim-label-pane> :text "Name:" :align 'left)
               (glim-get-frame-pane .frame. 'name-entry)))
       (----)
       (dialog-buttons
        (dialog-action-gadget "OK" 'ok)
        (dialog-action-gadget "Close" 'close))
       )))
  )

(define-method initialize ((self <node-creator-dialog>) initargs)
  (next-method self
               (list* :name 'node-creator
                      :pretty-name "Node creator"
                      initargs))
  )

;; node-creator-activate! callback class-constraint
(define (node-creator-activate! callback class)
  (let* ((self (instance-of <node-creator-dialog>))
         (selector (glim-get-frame-pane self 'class-selector))
         (entry (glim-get-frame-pane self 'name-entry)))
    (set! (ref self 'callback)  callback)
    (gtk-entry-set-text (widget-of entry) "")
    (class-selector-set-class self class)
    (glim-run-frame-top-level self)))

;;------------------------------------------------------------------
;; Actions
;;

;; Dialog action
(define-method dialog-action ((self <node-creator-dialog>) action)
  (case action
    ((ok)
     (let* ((entry (glim-get-frame-pane self 'name-entry))
            (name  (gtk-entry-get-text (widget-of entry)))
            (class (glim-gadget-value (glim-get-frame-pane self 'class-selector))))
       (when (valid-node-name? name)
         (cond ((ref self 'callback) => (cut <> class name)))
         (glim-frame-exit self))))
    (else 
     (glim-frame-exit self))))

;;------------------------------------------------------------------
;; Class selector
;;

(define (make-class-selector dialog)
  (make <glim-option-pane>
    :name-key (lambda (x) (x->string (class-name x)))))

(define (class-selector-set-class dialog class)
  (let ((f (glim-get-frame-pane dialog 'class-selector)))
    (set! (ref f 'items) (node-meta-all-concrete-subclasses class))))

(define (class-selector-value dialog)
  (glim-gadget-value (glim-get-frame-pane dialog 'class-selector)))

(provide "banyan/gui/nodecreator")
