;;;
;;; banyan/gui/nodeduplicator - node duplicate dialog
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: nodeduplicator.scm,v 1.2 2003/01/14 13:41:39 shirok Exp $
;;;

(define-module banyan.gui.nodeduplicator
  (extend banyan.gui)
  (export <node-duplicate-dialog>))
(select-module banyan.gui.nodeduplicator)

;;------------------------------------------------------------------
;; A dialog to duplicate selected instances.
;;

(define-glim-application-frame <node-duplicate-dialog> ()
  ()
  (:panes
   (duplicate-shader (make <glim-toggle-button>
                       :label "Duplicate shader"))
   (duplicate-object (make <glim-toggle-button>
                       :label "Duplicate object"))
   (push-selections (make <glim-toggle-button>
                      :label "Push original selections to the selection stack"))
   )
  (:layouts
   (default
     (glim-vertically (:spacing 2 :thickness 4)
       duplicate-shader
       duplicate-object
       (----)
       push-selections
       (----)
       (dialog-buttons
        (dialog-action-gadget "OK" 'ok)
        (dialog-action-gadget "Apply" 'apply)
        (dialog-action-gadget "Close" 'close))
       )))
  )

(define-method initialize ((self <node-duplicate-dialog>) initargs)
  (next-method self
               (list* :name 'node-duplicate
                      :pretty-name "Node duplicate"
                      initargs))
  )

;;------------------------------------------------------------------
;; Actions
;;

(define-method dialog-action ((self <node-duplicate-dialog>) action)
  (when (memq action '(ok apply))
    (let* ((scene (current-scene))
           (selected (selections-of scene))
           (dup-shader? (glim-gadget-value
                         (glim-get-frame-pane self 'duplicate-shader)))
           (dup-object? (glim-gadget-value
                         (glim-get-frame-pane self 'duplicate-object)))
           (push-sel?   (glim-gadget-value
                         (glim-get-frame-pane self 'push-selections)))
           (duped (map (cut instance-duplicate <>
                            :duplicate-shader dup-shader?
                            :duplicate-object dup-object?
                            :select-duplicated #f)
                       selected))
           )
      (when push-sel? (push-selections scene))
      (apply set-selection scene duped)
      ))
  (when (memq action '(ok close))
    (glim-frame-exit self)))

(provide "banyan/gui/nodeduplicator")
