;;;
;;; banyan/gui/partdisplay - a gadget to show the current part
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: partdisplay.scm,v 1.5 2003/01/14 10:01:52 shirok Exp $
;;;

(define-module banyan.gui.partdisplay
  (use gauche.mop.singleton)
  (use gauche.mop.validator)
  (use srfi-2)
  (extend banyan.gui)
  (export <part-display>))
(select-module banyan.gui.partdisplay)

;;==================================================================
;; Part display
;;

;; A labeled box that shows the current part.  Synchronized with
;; the change of the current part.    If :active? is true at the
;; initialization time, the boxed text area is a button to pop up
;; the part browser.

(define-class <part-display> (<glim-gadget>)
  ())

(define-method initialize ((self <part-display>) initargs)
  (let* ((w   (gtk-hbox-new #f 2))
         (lab (gtk-label-new "Current Part: "))
         (f   (gtk-frame-new #f))
         (active? (get-keyword :active? initargs #f))
         (txt ((if active? gtk-button-new-with-label gtk-label-new)
               (or (and-let* ((g (current-part))) (part-descriptive-name g))
                   "")))
         (txt-set! (if active?
                       (lambda (val)
                         (gtk-label-set-text (gtk-bin-get-child txt) val))
                       (lambda (val)
                         (gtk-label-set-text txt val))))
         )
    (gtk-box-pack-start w lab #f #f 2)
    (gtk-box-pack-start w f #t #t 2)
    (gtk-container-add f txt)
    (gtk-frame-set-shadow-type f GTK_SHADOW_ETCHED_IN)
    (when active?
      (gtk-button-set-relief txt GTK_RELIEF_NONE)
      (g-signal-connect txt "clicked"
                        (lambda _ (dialog-show <part-browser>)))
      )
    (add-hook! part-watchers
               (lambda (scene)
                 (txt-set! (part-descriptive-name (ref scene 'default-part)))))
    (add-hook! part-modified-watchers
               (lambda (part)
                 (and-let* ((scene (current-scene))
                            (cur-part (ref scene 'default-part))
                            ((eq? part cur-part)))
                   (txt-set! (part-descriptive-name part)))))
    (set! (widget-of self) w)
    (next-method)))

(provide "banyan/gui/partdisplay")
