;;;
;;; banyan/gui/repobrowser - repository browser
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: repobrowser.scm,v 1.11 2003/01/19 15:59:03 shirok Exp $
;;;

(define-module banyan.gui.repobrowser
  (use srfi-1)
  (use srfi-2)
  (use srfi-13)
  (use gauche.parameter)
  (extend banyan.gui)
  (export <repository-browser-dialog>))
(select-module banyan.gui.repobrowser)

;;==========================================================
;; Repository browser
;;   Show the parts in the current repository.
;;

(define-glim-application-frame <repository-browser-dialog> ()
  ()
  (:panes
   (repository-display  (make <repository-display>))
   (selector (make <glim-hierarchy-selector-pane>
               :dirlist-title "Subparts"
               :filelist-title "Parts"
               :value-changed-callback
               (lambda (pane path) (write path) (newline))
               :expand-hierarchy-callback
               (lambda (pane path)
                 (let1 parts (repository-parts (current-repository)
                                               (glim-hierarchy-path-string pane path))
                   (values (map car
                                (filter (lambda (e)
                                          (get-keyword :subdir (cdr e) #f))
                                        parts))
                           (map car
                                (filter (lambda (e)
                                          (get-keyword :part (cdr e) #f))
                                        parts))))))
             )
   (import-to-current (make <glim-toggle-button>
                        :label "Import into the current part"))
   )
  (:layouts
   (default
     (glim-vertically (:thickness 2 :spacing 2)
       repository-display
       (:expand selector)
       import-to-current
       (----)
       (dialog-buttons
        (dialog-action-gadget "OK" 'ok)
        (dialog-action-gadget "Close" 'close))
       )))
  )

(define-method initialize ((self <repository-browser-dialog>) initargs)
  (next-method self
               (list* :name 'repository-browser
                      :pretty-name "Repository Browser"
                      initargs))
  (parameter-observer-add! current-repository
                           (lambda (old repo)
                             (refresh-name-list self repo)))
  (refresh-name-list self (current-repository))
  )

(define-method refresh-name-list ((self <repository-browser-dialog>) repo)
  (when repo
    (let1 sel (glim-get-frame-pane self 'selector)
      (set! (glim-hierarchy-current-directory sel) '())
      (glim-refresh-view sel))))

;; NB: glim-hierarchy-selector-pane always returns an absolute path,
;; but for our purpose relative path is handy, hence string-drop.
(define-method repository-browser-get-path-string
    ((self <repository-browser-dialog>))
  (let1 sel (glim-get-frame-pane self 'selector)
    (string-drop 
     (glim-hierarchy-path-string sel (glim-gadget-value sel))
     1)))

;; Action
(define-method dialog-action ((self <repository-browser-dialog>) type)
  (case type
    ((ok)
     (with-error-handler
         (lambda (e)
           (error-dialog "~a" (ref e 'message)))
       (lambda ()
         (and-let* ((path (repository-browser-get-path-string self))
                    (parts (repository-check-out (current-repository) path)))
           (when (glim-gadget-value (glim-get-frame-pane self 'import-to-current))
             (push! (ref (current-part) 'imports) (car parts))
             (set-modified! (current-part)))
           (add-parts (current-scene) parts)
           (viewports-post-redisplay)
           (glim-frame-exit self)))))
    (else
     (glim-frame-exit self))))

(provide "banyan/gui/repobrowser")
