;;;
;;; banyan/gui/shelf - shelf
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: shelf.scm,v 1.3 2003/12/17 10:50:09 shirok Exp $
;;;

(define-module banyan.gui.shelf
  (extend banyan.gui)
  (export <composer-shelf>)
  )
(select-module banyan.gui.shelf)

;; Shelf for composer application window.
;; For now, the items in the shelf is fixed.

;; In future, the following feature should be added:
;;   - ability to add/remove icons
;;   - ability to switch orientation

(define-class <composer-shelf> (<glim-gtk-mixin> <gtk-vbox>)
  ())

(define-method initialize ((self <composer-shelf>) initargs)
  (next-method)
  (shelf-populate self))

(define-method shelf-populate ((self <composer-shelf>))
  (define (manip-icon file mode)
    (let1 button (image-button-new file)
      (gtk-box-pack-start self button #f #f 0)
      (g-signal-connect button "clicked"
                        (lambda _
                          (manipulator-mode-set! mode)
                          (viewports-post-redisplay)))))
  (define (shape-icon file class)
    (let1 button (image-button-new file)
      (gtk-box-pack-start self button #f #f 0)
      (when class
        (g-signal-connect button "clicked"
                          (lambda _
                            (glim-run-frame-top-level (instance-of class)))))
      ))

  (manip-icon "icon-select.png" #f)
  (manip-icon "icon-translation.png" 'translate)
  (manip-icon "icon-rotation.png" 'rotate)
  (manip-icon "icon-scale.png" 'scale)
  (gtk-box-pack-start self (gtk-hseparator-new) #f #f 5)
  (shape-icon "icon-cube.png" <cube-creator-dialog>)
  (shape-icon "icon-sphere.png" <sphere-creator-dialog>)
  (shape-icon "icon-cylinder.png" <cylinder-creator-dialog>)
  (shape-icon "icon-cone.png" <cone-creator-dialog>)
  (shape-icon "icon-plane.png" <plane-creator-dialog>)
  (shape-icon "icon-lambda.png" <procedural-creator-dialog>)
  )

(provide "banyan/gui/shelf")
