;;;
;;; banyan.init - application initialization
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: init.scm,v 1.2 2003/01/19 02:05:52 shirok Exp $
;;;

(select-module banyan)

;; This module reads common initialization files and sets up
;; some parameters.

;;--------------------------------------------------------------
;; User customization file (~/.banyanrc)
;;
;; Special forms:
;;
;;   (repository <url> &keyword user password default)
;;
;;       Adds <url> to the pull-down list of repository.
;;       USER is used as the login name of the repository.
;;       If omitted, the current user's login name is used.
;;       PASSWORD is, if given, used as the password for the
;;       repository.  If PASSWORD is omitted and USER is not
;;       "guest", the application program must ask the user
;;       the password.   To indicate empty password, specify "".
;;       If DEFAULT is #t, use it as the default repository.
;;

;; .banyanrc file is loaded in banyan.init module, to avoid
;; inadvertent name collision.
(define-module banyan.init
  (use file.util)
  (define *repositories* '())
  (define (repository url . args)
    (push! *repositories* (cons url args)))
  )

;; TODO: allow reloading .banyanrc
(define banyan-init
  (let ((initialized #f))
    (lambda (args)
      (unless initialized
        (let ((rcfile (build-path (home-directory) ".banyanrc")))
          (when (file-exists? rcfile)
            (eval `(load ,rcfile) (find-module 'banyan.init))))
        (let* ((repospecs (reverse (with-module banyan.init *repositories*)))
               ;; NB: adds local repository if not given.
               (local-repo #`"file:,(home-directory)/banyan/repository")
               (repos (map (lambda (specs) (apply open-repository specs))
                           (if (assoc local-repo repospecs)
                               repospecs
                               (cons (list local-repo) repospecs)))))
          (standard-repositories repos)
          (current-repository (or (find (cut ref <> 'default) repos)
                                  (car repos)))
          )))
    ))

(provide "banyan/init")




