;;;
;;; banyan/light - lights and light groups
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: light.scm,v 1.10 2003/01/04 14:34:30 shirok Exp $
;;;

(select-module banyan)

;;;============================================================
;;; Light base class
;;;
(define-class <light> (<updatable-node>)
  ((transform  :init-keyword :transform :accessor transform-of
               :initform (make <trs-transform>)
               :validator (lambda (o v)
                            (unless (is-a? v <transform>)
                              (error "transform required, but got %S" v))
                            (slot-set! (%c-node o) 'transform (%c-node v))
                            v))
   )
  :abstract #t
  :c-class <%light>)

(define-class <gl-light> (<light>)
  ((ambient    :allocation :c-node
               :init-keyword :ambient :accessor ambient-of)
   (diffuse    :allocation :c-node
               :init-keyword :diffuse :accessor diffuse-of)
   (specular   :allocation :c-node
               :init-keyword :specular :accessor specular-of)
   (spot-exponent :allocation :c-node
                  :init-keyword :spot-exponent :accessor spot-exponent-of)
   (spot-cutoff   :allocation :c-node
                  :init-keyword :spot-cutoff :accessor spot-cutoff-of)
   (constant-attenuation  :allocation :c-node
                          :init-keyword :constant-attenuation
                          :accessor constant-attenuation-of)
   (linear-attenuation    :allocation :c-node
                          :init-keyword :linear-attenuation
                          :accessor linear-attenuation-of)
   (quadratic-attenuation :allocation :c-node
                          :init-keyword :quadratic-attenuation
                          :accessor quadratic-attenuation-of)
   )
  :c-class <%light>)

(define-method node-depends ((self <light>))
  (let1 xf (transform-of self)
    (if xf (list xf) '())))

;;;============================================================
;;; Light group
;;;

(define-class <light-group> (<renderable-node>)
  ((lights :allocation :c-node :accessor lights-of))
  :c-class <%light-group>)

(define-method node-render ((self <light-group>) mode)
  (%light-group-render (%c-node self) mode))

;; Default light group
(define default-light-group
  (with-part (transient-part)
    (let ((lg (make <light-group>))
          (tr (make <trs-transform>
                :translation (vector4f 10.0 10.0 10.0))))
      (set! (ref (lights-of lg) 0)
            (%c-node (make <gl-light>
                       :transform tr
                       :diffuse  (vector4f 1.0 1.0 1.0 1.0)
                       :specular (vector4f 1.0 1.0 1.0 1.0))))
      lg)))

(provide "banyan/light")
