;;;
;;; banyan.util - utilities
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: util.scm,v 1.13 2003/02/17 23:07:32 shirok Exp $
;;;

(select-module banyan)

;; This file defines general utility procedures.
;; Most of them should eventually be included in the Gauche distribution.

(define-syntax unwind-protect
  (syntax-rules ()
    ((_ body finally)
     (with-error-handler
         (lambda (e) finally (raise e))
       (lambda ()
         (begin0 body finally))))
    ))

(define (call-with-temporary-output-file file proc . opts)
  (let-keywords* opts ((backup-suffix ".orig"))
    (receive (outport tmpfile) (sys-mkstemp file)
      (begin0
       (apply call-with-output-file tmpfile proc opts)
       (if (file-exists? file)
           (let ((operm (file-perm file)))
             (sys-chmod tmpfile operm)
             (sys-rename file (string-append file backup-suffix))
             (sys-rename tmpfile file))
           (begin
             (sys-chmod tmpfile #o644)
             (sys-rename tmpfile file)))))))

(define (set-keyword key lis val)
  (let loop ((l lis) (r '()))
    (cond ((null? l) (list* key val (reverse! r)))
          ((null? (cdr l)) (error "keyword list not even" lis))
          ((eqv? (car l) key)
           (append! (reverse! r) (list* key val (cddr l))))
          (else (loop (cddr l) (list* (cadr l) (car l) r))))))

(define (forest->trees trees walker tree=?)
  (define tlist '())

  (define (reg tree)
    (cond ((assoc tree tlist tree=?) => (cut set-cdr! <> #f))
          (else (push! tlist (cons tree #f))))
    (walker reg tree))
  
  (for-each (lambda (tree)
              (unless (assoc tree tlist tree=?)
                (push! tlist (cons tree #t))
                (walker reg tree)))
            trees)

  (filter-map (lambda (e) (and (cdr e) (car e))) tlist))

;; Quaternion -> axis and angle.  This should eventually be in Gauche-gl.
(define (quatf->rotation q)
  (receive (f t r h s) (matrix4f-decompose (quatf->matrix4f q))
    (if f
        (matrix4f->rotation r)
        (vector4f 0 0 1 0))))

;; RGB -> luminance.  This should also be in Gauche-gl?
(define (rgb->luminance rgb)
  (+ (* (ref rgb 0) 0.3) (* (ref rgb 1) 0.59) (* (ref rgb 2) 0.11)))

(provide "banyan/util")
