;;;
;;; glim/pane.scm - panes
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: pane.scm,v 1.14 2003/01/01 00:59:50 shirok Exp $
;;;

(define-module glim.pane
  (use srfi-1)
  (use gtk)
  (use gauche.mop.validator)
  (export glim-gtk-widget->pane widget-of 
          glim-gadget-value |setter of glim-gadget-value|
          glim-refresh-view 

          <glim-pane> <glim-gadget> <glim-compound-gadget>
          <glim-value-gadget> <glim-action-gadget>
          <glim-orientation-gadget-mixin> <glim-labelled-gadget-mixin>
          <glim-label-pane> <glim-push-button> <glim-toggle-button>
          <glim-radio-box> <glim-check-box> <glim-list-pane> <glim-option-pane>
          <glim-combo-box> <glim-spin-button>
          <glim-vbox-pane> <glim-hbox-pane> <glim-table-pane> 
          <glim-spacing-pane> <glim-outlined-pane> <glim-scroller-pane>
          
          glim-activate-callback glim-value-changed-callback
          glim-gadget-orientation

          %glim-canonicalize-content-form
          glim-horizontally glim-vertically glim-tabling
          glim-spacing glim-labelling glim-outlining glim-scrolling

          <glim-hierarchy-selector-mixin> <glim-hierarchy-selector-pane>
          glim-hierarchy-current-directory glim-hierarchy-path-string
          glim-hierarchy-parse-path-string
          )
  )
(select-module glim.pane)

;; NB: for now, I associate gtk widget directly to the pane class
;; In future, they have to switch implementation class depends on
;; the mode (e.g. GTk vs Web form)

;; Pane base class.  This is an abstract class.
(define-class <glim-pane> ()  ())

(define-method glim-activate-callback ((self <glim-pane>)) #f)
(define-method glim-value-changed-callback ((self <glim-pane>) value) #f)

(define-method glim-refresh-view ((self <glim-pane>)) #f)

;; <glim-gadget> 
;;    The implementation can either multi-inherit <glim-gadget> and
;;    gtk-widget class, or keep gtk-widget instance in the slot.
;;    Either way, widget-of should always return the valid widget.
;;    (NB: watch out the dependency of slot initialization; "widget" slot
;;    must be initialized before any initialization routine that requires
;;    widget runs.)

(define-class <glim-gadget> (<glim-pane>)
  ((%widget :init-keyword :widget)
   ))
(define-method widget-of ((w <glim-gadget>)) (slot-ref w '%widget))
(define-method (setter widget-of) ((w <glim-gadget>) value)
  (slot-set! w '%widget value))

;; This mixin is used to turn gtk widget into glim pane.  Should be
;; used in superclass list as 
;;    (<glim-gtk-mixin> <...glim class...> <...gtk class...>
(define-class <glim-gtk-mixin> ()
  ())
(define-method widget-of ((w <glim-gtk-mixin>)) w)
(define-method (setter widget-of) ((w <glim-gtk-mixin>) value)
  (error "can't set widget to <glim-gtk-mixin> instance" w))

;; This allows bare gtk-widget to be used in place of pane,
;; although it is not encouraged.
(define-method widget-of ((w <gtk-widget>)) w)
(define glim-gtk-widget->pane identity)

;;=============================================================
;; Leaf Panes
;;

;; value gadget
(define-class <glim-value-gadget> (<glim-gadget>)
  ((value-changed-callback :init-keyword :value-changed-callback
                           :init-value #f)
   (value :allocation :virtual :init-keyword :value
          :slot-ref  (lambda (o) (glim-gadget-value o))
          :slot-set! (lambda (o v) (set! (glim-gadget-value o) v)))
   ))

(define-method glim-gadget-value ((gadget <glim-value-gadget>))
  ;; subclass should implement this
  #f)

(define-method (setter glim-gadget-value) ((gadget <glim-value-gadget>) value)
  ;; subclass should provide the appropriate method
  #f)

(define-method glim-value-changed-callback ((gadget <glim-value-gadget>) value)
  (cond ((slot-ref gadget 'value-changed-callback) => (cut <> gadget value))
        (else (next-method))))

(define-class <glim-action-gadget> (<glim-gadget>)
  ((activate-callback :init-value #f :init-keyword :activate-callback)
   ))

(define-method glim-activate-callback ((gadget <glim-action-gadget>))
  (cond ((slot-ref gadget 'activate-callback) => (cut <> gadget))
        (else (next-method))))

(define-class <glim-orientation-gadget-mixin> ()
  ())

(define-method glim-gadget-orientation ((self <glim-orientation-gadget-mixin>))
  :horizontal)

(define-class <glim-labelled-gadget-mixin> ()
  ())

;; Autoloads
(autoload "glim/pane/label"  <glim-label-pane>)
(autoload "glim/pane/button" <glim-push-button> <glim-toggle-button>)
(autoload "glim/pane/list"   <glim-list-pane>)
(autoload "glim/pane/option" <glim-option-pane>)
(autoload "glim/pane/spin"   <glim-spin-button>)

;;=============================================================
;; Layout Panes
;;

(define-class <glim-layout-pane> (<glim-gadget>)
  ((contents :init-keyword :contents :initform '())))

(define (%get-pane-maker z)
  (cond ((symbol? z) `(glim-get-frame-pane .frame. ',z))
        ((list? z) z)
        (else (error "bad content specification in layout" z))))

(define (%glim-canonicalize-content-form contents)
  (define (canon-pane content)
    (cond ((symbol? content)
           `(list #f ,(%get-pane-maker content)))
          ((not (list? content))
           (error "bad content specification in layout" content))
          ((keyword? (car content))
           `(list ,(car content) ,(%get-pane-maker (cadr content))))
          (else `(list #f ,content))))
  (cons list (map canon-pane contents)))

;; Autoloads
(autoload "glim/pane/box"
          <glim-hbox-pane> <glim-vbox-pane>
          (:macro glim-horizontally) (:macro glim-vertically)
          (:macro glim-labelling))
(autoload "glim/pane/frame"
          <glim-outlined-pane> (:macro glim-outlining)
          <glim-spacing-pane>  (:macro glim-spacing))
(autoload "glim/pane/scroller"
          <glim-scroller-pane> (:macro glim-scrolling))
(autoload "glim/pane/table"
          <glim-table-pane>    (:macro glim-tabling))

;;===============================================================
;; Composite panes
;;

(autoload "glim/pane/hselect"
          <glim-hierarchy-selector-mixin>
          <glim-hierarchy-selector-pane>
          glim-hierarchy-current-directory
          glim-hierarchy-path-string
          glim-hierarchy-parse-path-string)

(provide "glim/pane")

