;;;
;;; glim/pane/box.scm - box layout panes, to be autoloaded by glim.pane
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: box.scm,v 1.7 2003/01/01 01:03:59 shirok Exp $
;;;

(select-module glim.pane)

;;-------------------------------------------------------------
;; <glim-hbox-pane> and <glim-vbox-pane>
;;

;; abstract class for common ops of hbox and vbox.
(define-class <glim-box-pane> (<glim-layout-pane>)
  ())

;; assumes the widget slot is already set
(define-method initialize ((self <glim-box-pane>) initargs)
  (let-keywords* initargs ((width -1)
                           (height -1)
                           (spacing 0)
                           (contents '()))
    (let ((box self))
      (when (or (positive? width) (positive? height))
        (gtk-widget-set-size-request box width height))
      (for-each (lambda (content)
                  (let* ((opt  (car content))
                         (pane (cadr content))
                         (pkopts
                          (case opt
                            ((#f)      (list #f #f spacing))
                            ((:fill)   (list #t #f spacing))
                            ((:expand) (list #t #t spacing))
                            (else (error "bad allocation spec" opt)))))
                    (apply gtk-box-pack-start box (widget-of pane) pkopts)))
                contents)))
  (next-method))

(define-class <glim-hbox-pane> (<glim-gtk-mixin> <glim-box-pane> <gtk-hbox>)
  ())

(define-method initialize ((self <glim-hbox-pane>) initargs)
  (let-keywords* initargs ((homogeneous #f)
                           (spacing     0)
                           (thickness   0))
    (next-method self (list* :homogeneous (if homogeneous 1 0)
                             :spacing spacing
                             initargs))
    (gtk-container-set-border-width self thickness)
    ))

(define-class <glim-vbox-pane> (<glim-gtk-mixin> <glim-box-pane> <gtk-vbox>)
  ())

(define-method initialize ((self <glim-vbox-pane>) initargs)
  (let-keywords* initargs ((homogeneous #f)
                           (spacing     0)
                           (thickness   0))
    (next-method self (list* :homogeneous (if homogeneous 1 0)
                             :spacing spacing
                             initargs))
    (gtk-container-set-border-width self thickness)
    ))

(define-macro (glim-horizontally options . contents)
  `(make <glim-hbox-pane> ,@options
         :contents ,(%glim-canonicalize-content-form contents)))

(define-macro (glim-vertically options . contents)
  `(make <glim-vbox-pane> ,@options
         :contents ,(%glim-canonicalize-content-form contents)))

;; labelled - use box pane
(define-macro (glim-labelling options content)
  (let* ((l (gensym))
         (p (gensym))
         (text  (get-keyword :label options ""))
         (align (get-keyword :label-alignment options :top))
         (labelw (get-keyword :label-width options -1))
         (labelh (get-keyword :label-width options -1))
         (box-maker (case align
                      ((:top :bottom) 'glim-vertically)
                      ((:left :right) 'glim-horizontally)
                      (else (error "bad value for label-alignment" align))))
         )
    `(let* ((,l (gtk-label-new ,text))
            (,p (,box-maker ,options
                            (glim-gtk-widget->pane ,l)
                            (:expand ,content)))
            )
       (gtk-widget-set-size-request ,l ,labelw ,labelh)
       ,p)))

(provide "glim/pane/box")

