;;;
;;; glim/pane/frame.scm - outlined and spacing pane, to be autoloaded.
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: frame.scm,v 1.7 2003/01/01 01:04:00 shirok Exp $
;;;

(select-module glim.pane)

;;-------------------------------------------------------------
;; <glim-outlined-pane>
;;

;; outlining -> gtk-frame
;;   :thickness
;;   :style
;;   :label
;;   :label-align

(define-class <glim-outlined-pane> (<glim-gtk-mixin>
                                    <glim-layout-pane>
                                    <gtk-frame>)
  ())

(define-method initialize ((self <glim-outlined-pane>) initargs)
  (next-method self '())
  (let-keywords* initargs ((style  :etched-in)
                           (width  -1)
                           (height -1)
                           (thickness 0)
                           (label  #f)
                           (label-align '(0.1 0.1))
                           (contents '()))
    (unless (= (length contents) 1)
      (error "outlining requires exactly one child" contents))
    (gtk-container-add self (widget-of (car contents)))
    (gtk-container-set-border-width self thickness)
    (gtk-widget-set-size-request self width height)
    (gtk-frame-set-label self label)
    (gtk-frame-set-label-align self (car label-align) (cadr label-align))
    (gtk-frame-set-shadow-type
     self
     (case style
       ((#f :none) GTK_SHADOW_NONE)
       ((:in)  GTK_SHADOW_IN)
       ((:out) GTK_SHADOW_OUT)
       ((:etched-in) GTK_SHADOW_ETCHED_IN)
       ((:etched-out) GTK_SHADOW_ETCHED_OUT)
       (else (error "bad style option for outlining" style))))
    ))

(define-macro (glim-outlining options content)
  `(make <glim-outlined-pane> ,@options
         :contents (list ,(%get-pane-maker content))))

;; spacing is just an gtkframe with :none style.
(define-macro (glim-spacing options content)
  `(make <glim-outlined-pane>
     :style :none ,@options
     :contents (list ,(%get-pane-maker content))))

(provide "glim/pane/frame")

