;;;
;;; glim/pane/list.scm - glim-list-pane, to be autoloaded by glim.pane
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: list.scm,v 1.6 2003/01/01 01:04:00 shirok Exp $
;;;

(select-module glim.pane)

;;---------------------------------------------------
;; <glim-list-pane>
;;
;;  NB: the order of slot of <glim-list-pane> is somewhat important,
;;  for the setter of items requires name-key slot is initialized beforehand.

(define (glim-list-update-mode list-pane value)
  (let ((sel  (gtk-tree-view-get-selection (ref list-pane 'w-view))))
    (gtk-tree-selection-set-mode sel
                                 (case value
                                   ((:exclusive) GTK_SELECTION_SINGLE)
                                   ((:nonexclusive) GTK_SELECTION_MULTIPLE)
                                   (else (error "invalid value for mode slot of <gtk-list-pane>: must be either :exclusive or :nonexclusive, but got" value)))))
  value)

(define (glim-list-update-items list-pane value)
  (let ((store (ref list-pane 'w-store))
        (name-key (ref list-pane 'name-key)))
    (gtk-list-store-clear store)
    (for-each (lambda (item)
                (gtk-list-store-set-value store
                                          (gtk-list-store-append store)
                                          0
                                          (name-key item)))
              value))
  value)

(define (glim-list-update-scroll-bars list-pane value)
  (let ((w (widget-of list-pane)))
    (apply gtk-scrolled-window-set-policy w
           (case value
             ((:both) (list GTK_POLICY_AUTOMATIC GTK_POLICY_AUTOMATIC))
             ((:horizontal) (list GTK_POLICY_AUTOMATIC GTK_POLICY_NEVER))
             ((:vertical)   (list GTK_POLICY_NEVER GTK_POLICY_AUTOMATIC))
             ((#f) (list GTK_POLICY_NEVER GTK_POLICY_NEVER))
             (else (error "invalid value for scroll-bars slot of <glim-list-pane>: must be either #f, :horizontal, :vertical or :both, but got" value)))))
  value)

(define-class <glim-list-pane> (<glim-gtk-mixin>
                                <glim-value-gadget>
                                <gtk-scrolled-window>)
  ((mode        :init-keyword :mode  :init-value :exclusive
                :validator glim-list-update-mode)
   (name-key    :init-keyword :name-key :init-value x->string)
   (value-key   :init-keyword :value-key :init-value identity)
   (items       :init-keyword :items :init-value '()
                :validator glim-list-update-items)
   (scroll-bars :init-keyword :scroll-bars :init-value :vertical
                :validator glim-list-update-scroll-bars)
   (w-view)  ;treeview
   (w-store) ;liststore
   )
  :metaclass <validator-meta>)

(define-method initialize ((self <glim-list-pane>) initargs)
  (let* ((store  (gtk-list-store-new <string>))
         (view   (gtk-tree-view-new-with-model store))
         (renderer (gtk-cell-renderer-text-new))
         (column (gtk-tree-view-column-new-with-attributes "selections"
                                                           renderer
                                                           "text" 0))
         )
    (gtk-tree-view-append-column view column)
    (gtk-tree-view-set-headers-visible view #f)
    (set! (ref self 'w-view) view)
    (set! (ref self 'w-store) store)
    (next-method)
    (gtk-scrolled-window-add-with-viewport self view)
    (when (null? (ref self 'items))
      (gtk-widget-set-size-request view 0 0)) ;avoid having error on empty list
    ))

(define-method glim-gadget-value ((self <glim-list-pane>))
  (define (iter->value iter)
    (and iter
         (let ((name (gtk-tree-model-get-value (ref self 'w-store) iter 0))
               (name-key (ref self 'name-key)))
           (find (lambda (v) (equal? (name-key v) name)) (ref self 'items)))))
  (if (eq? (ref self 'mode) :exclusive)
      (iter->value
       (gtk-tree-selection-get-selected
        (gtk-tree-view-get-selection (ref self 'w-view))))
      (map iter->value
           (gtk-tree-selection-get-selected-multi
            (gtk-tree-view-get-selection (ref self 'w-view))))))

(define-method (setter glim-gadget-value) ((self <glim-list-pane>) value)
  (error "sorry, not implemented yet") (values))

(provide "glim/pane/list")

