;;;
;;; glim/pane/option.scm - glim-option-pane, to be autoloaded by glim.pane
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: option.scm,v 1.7 2003/01/01 01:04:00 shirok Exp $
;;;

(select-module glim.pane)

;;---------------------------------------------------
;; <glim-option-pane>
;;
;;  NB: the order of slot of <glim-option-pane> is somewhat important,
;;  for the setter of items requires name-key slot is initialized beforehand.

(define (glim-option-update-items option-pane value)
  (let ((omenu (ref option-pane 'w-menu))
        (menu  (gtk-menu-new))
        (name-key (ref option-pane 'name-key)))
    (gtk-option-menu-remove-menu option-pane)
    (gtk-option-menu-set-menu option-pane menu)
    (when omenu (gtk-widget-destroy omenu))
    (set! (ref option-pane 'w-menu) menu)
    (for-each (lambda (v)
                (let1 item (gtk-menu-item-new-with-label (name-key v))
                  (gtk-widget-show item)
                  (gtk-menu-shell-append menu item)))
              value)
    value))

(define-class <glim-option-pane> (<glim-gtk-mixin>
                                  <glim-value-gadget>
                                  <gtk-option-menu>)
  ((name-key    :init-keyword :name-key :init-value x->string)
   (value-key   :init-keyword :value-key :init-value identity)
   (default     :init-keyword :default :init-value #f)
   (test        :init-keyword :test :init-value eqv?)
   (items       :init-keyword :items :init-value '()
                :validator glim-option-update-items
                :observer (lambda (o v)
                            (unless (null? v)
                              (gtk-option-menu-set-history o 0))))
   (w-menu)
   )
  :metaclass <validator-meta>)

(define-method initialize ((self <glim-option-pane>) initargs)
  (set! (ref self 'w-menu) #f) ;; will be set by glim-option-update-items
  (next-method)
  (g-signal-connect self "changed"
                    (lambda (w)
                      (glim-value-changed-callback
                       self
                       (glim-gadget-value self))))
  )

(define-method glim-gadget-value ((self <glim-option-pane>))
  (let1 sel (gtk-option-menu-get-history self)
    (if (>= sel 0)
        ((ref self 'value-key) (list-ref (ref self 'items) sel))
        (ref self 'default))))

(define-method (setter glim-gadget-value) ((self <glim-option-pane>)
                                           value)
  (let* ((test (ref self 'test))
         (sel  (list-index (cut test <> value) (ref self 'items))))
    (unless sel
      (errorf "invalid option value ~s for option-pane ~s" value self))
    (gtk-option-menu-set-history self sel)))

(provide "glim/pane/option")

