;;;
;;; glim/pane/scroller.scm - scrolled panes, to be autoloaded
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: scroller.scm,v 1.5 2003/01/01 01:04:00 shirok Exp $
;;;

(select-module glim.pane)

;;-------------------------------------------------------------
;; <glim-scroller-pane>
;;

(define-class <glim-scroller-pane> (<glim-gtk-mixin>
                                    <glim-layout-pane>
                                    <gtk-scrolled-window>)
  ())

(define-method initialize ((self <glim-scroller-pane>) initargs)
  (let-keywords* initargs ((width  -1)
                           (height -1)
                           (thickness #f)
                           (scroll-bar #f)
                           (contents '()))
    (let ((policy
           (case scroll-bar
             ((:both) (list GTK_POLICY_AUTOMATIC GTK_POLICY_AUTOMATIC))
             ((:horizontal) (list GTK_POLICY_AUTOMATIC GTK_POLICY_NEVER))
             ((:vertical)   (list GTK_POLICY_NEVER GTK_POLICY_AUTOMATIC))
             ((#f) (list GTK_POLICY_NEVER GTK_POLICY_NEVER))
             (else (error "invalid value for scroll-bars slot of glim-scrolling: must be either #f, :horizontal, :vertical or :both, but got" value)))))
      (unless (= (length contents) 1)
        (error "scrolling layout requires exactly one child" contents))
      (next-method)
      (gtk-scrolled-window-add-with-viewport self (widget-of (car contents)))
      (apply gtk-scrolled-window-set-policy self policy)
      (gtk-widget-set-size-request self width height)
      (when thickness
        (gtk-container-set-border-width self thickness))
      )))

(define-macro (glim-scrolling options content)
  `(make <glim-scroller-pane> ,@options
         :contents (list ,(%get-pane-maker content))))

(provide "glim/pane/scroller")

