;;;
;;; glim/pane/table.scm - table layout, to be autoloaded.
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: table.scm,v 1.6 2003/01/01 01:04:00 shirok Exp $
;;;

(select-module glim.pane)

;;-------------------------------------------------------------
;; <glim-table-pane>
;;

(define-class <glim-table-pane> (<glim-gadget>)
  ())

(define-method initialize ((pane <glim-table-pane>) initargs)
  (let-keywords* initargs ((contents '())
                           (homogeneous #f)
                           (spacing   0)
                           (x-spacing spacing)
                           (y-spacing spacing)
                           (row-options '())
                           (column-options '()))
    (define (get-cell-option options count max)
      (cond ((find (lambda (spec)
                     (let ((nums (car spec)))
                       (or (and (integer? nums) (= nums count))
                           (and (list? nums)  (memv count muns))
                           (and (eq? nums '$) (= count (- max 1))))))
                   options)
             => (lambda (spec)
                  (logior (if (memv :expand (cdr spec)) GTK_EXPAND 0)
                          (if (memv :fill   (cdr spec)) GTK_FILL 0)
                          (if (memv :shrink (cdr spec)) GTK_SHRINK 0))))
            (else 0)))
    (let* ((rows     (length contents))
           (cols     (apply max (map length contents)))
           (table    (gtk-table-new rows cols homogeneous)))
      (set! (widget-of pane) table)
      (next-method)
      (let row-loop ((rowcount 0)
                     (row contents))
        (unless (null? row)
          (let col-loop ((colcount 0)
                         (col (car row)))
            (cond ((null? col)
                   (row-loop (+ rowcount 1) (cdr row)))
                  (else
                   (gtk-table-attach table
                                     (widget-of (car col))
                                     colcount (+ colcount 1)
                                     rowcount (+ rowcount 1)
                                     (get-cell-option column-options colcount cols)
                                     (get-cell-option row-options rowcount rows)
                                     x-spacing y-spacing)
                   (col-loop (+ colcount 1) (cdr col))))
            )))
      )))

(define-macro (glim-tabling options . rows)
  `(make <glim-table-pane> ,@options
     :contents (list . ,rows)))

(provide "glim/pane/table")

