;;
;; test for glim module
;;  $Id: test-glim.scm,v 1.1 2002/12/24 09:58:20 shirok Exp $

;; This script tries to issue some basic calls of glim.
;; This test can check neither if the calls produce expected
;; visual representation, nor if the created objects respond
;; to user input properly---I don't have test framework for
;; such tasks yet---but at least these test will catch obvious
;; mistakes.

(use gauche.test)
(add-load-path ".")

(test-start "GLIM")

(use gtk)
(use glim)

(gtk-init '("test-glim"))

(define the-frame #f)

(define-glim-application-frame <foo> ()
  ((slot1 :init-value 3))
  (:panes
   (label (make <glim-label-pane> :text "FooLabel")))
  (:layouts
   (default
     (glim-vertically ()
       label)))
  )

(define the-foo #f)

(test "app-frame" #t
      (lambda ()
        (let ((a (instance-of <foo>)))
          (set! the-foo a)
          (and (eqv? (ref a 'slot1) 3)
               (is-a? (glim-get-frame-pane a 'label) <glim-label-pane>)
               (equal? (ref (glim-get-frame-pane a 'label) 'text)
                       "FooLabel")
               ))))

(test "singleton app-frame" #t
      (lambda ()
        (and (eq? (instance-of <foo>) the-foo)
             (not (eq? (make <foo>) the-foo)))))

;; creates many kind of panes
(define-glim-application-frame <bar> ()
  ()
  (:panes
   ;; single gadgets
   (label  (make <glim-label-pane>))
   (push   (make <glim-push-button>))
   (toggle (make <glim-toggle-button>))
   (list   (make <glim-list-pane>))
   (option (make <glim-option-pane>))
   (spin   (make <glim-spin-button>))
   ;; layout panes
   (vertical   (glim-vertically () (make <glim-label-pane>)))
   (horizontal (glim-horizontally () (make <glim-label-pane>)))
   (labelled   (glim-labelling () (make <glim-label-pane>)))
   (outlined   (glim-outlining () (make <glim-label-pane>)))
   (spaced     (glim-spacing () (make <glim-label-pane>)))
   (tabular    (glim-tabling ()
                 (list (make <glim-label-pane>)
                       (make <glim-label-pane>)
                       (make <glim-label-pane>))
                 (list (make <glim-label-pane>)
                       (make <glim-label-pane>)
                       (make <glim-label-pane>))
                 (list (make <glim-label-pane>)
                       (make <glim-label-pane>)
                       (make <glim-label-pane>))
                 ))
   )
  (:layouts
   (default
     (glim-vertically ()
       (glim-horizontally ()
         label push toggle list option spin)
       (glim-horizontally ()
         vertical horizontal labelled outlined spaced tabular))))
  )

(test "complex app-frame" #t
      (lambda ()
        (let ((a (instance-of <bar>)))
          (glim-application-frame? a))))


(test-end)

               

        

