;;;
;;; x3d/vrml - VRML node definitions
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: vrml.scm,v 1.4 2003/02/21 05:33:41 shirok Exp $
;;;

(define-module x3d.vrml
  (use x3d.field)
  (use x3d.node)
  (export-all))
(select-module x3d.vrml)

;;-------------------------------------------------------------------
;; VRML classes
;;

(define-vrml-class <Anchor> ()
  ((children        :exposed-field (<MFNode> "children"))
   (description     :exposed-field (<SFString> "description" ""))
   (parameter       :exposed-field (<MFString> "parameter" '()))
   (url             :exposed-field (<MFString> "url" '()))
   (bbox-center     :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size       :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   ;; event I/O
   (add-children    :event-in      (<MFNode> "addChildren"))
   (remove-children :event-in      (<MFNode> "removeChildren"))
   ))

(define-vrml-class <Appearance> ()
  ((material         :exposed-field (<SFNode> "material" #f))
   (texture          :exposed-field (<SFNode> "texture" #f))
   (textureTransform :exposed-field (<SFNode> "textureTransform" #f))
   ))

(define-vrml-class <AudioClip> ()
  ((description      :exposed-field (<SFString> "description" ""))
   (loop             :exposed-field (<SFBool> "loop" #f))
   (pitch            :exposed-field (<SFFloat> "pitch" 1.0))
   (start-time       :exposed-field (<SFTime> "startTime" 0))
   (stop-time        :exposed-field (<SFTime> "stopTime" 0))
   (url              :exposed-field (<MFString> "url" '()))
   ;; Event I/O
   (duration-changed :event-out     (<SFTime> "duration_changed"))
   (is-active        :event-out     (<SFBool> "isActive"))
   ))

(define-vrml-class <Background> ()
  ((ground-angle     :exposed-field (<MFFloat> "groundAngle" '()))
   (ground-color     :exposed-field (<MFColor> "groundColor" '()))
   (back-url         :exposed-field (<MFString> "backUrl" '()))
   (bottom-url       :exposed-field (<MFString> "bottomUrl" '()))
   (front-url        :exposed-field (<MFString> "frontUrl" '()))
   (left-url         :exposed-field (<MFString> "leftUrl" '()))
   (right-url        :exposed-field (<MFString> "rightUrl" '()))
   (top-url          :exposed-field (<MFString> "topUrl" '()))
   (sky-angle        :exposed-field (<MFFloat> "styAngle" '()))
   (sky-color        :exposed-field (<MFColor> "skyColor" '(#(0 0 0))))
   ;; event i/o
   (set-bind         :event-in      (<SFBool> "set_bind"))
   (is-bound         :event-out     (<SFBool> "isBound"))
   ))

(define-vrml-class <Billboard> ()
  ((axis-of-rotation :exposed-field (<SFVec3f> "axisOfRotation" '#(0 1 0)))
   (children         :exposed-field (<MFNode> "children" '()))
   (bbox-center      :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size        :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   ;; event I/O
   (add-children     :event-in      (<MFNode> "addChildren"))
   (remove-children  :event-in      (<MFNode> "removeChildren"))
   ))

(define-vrml-class <Box> ()
  ((size             :field         (<SFVec3f> "size" '#(2 2 2)))
   ))

(define-vrml-class <Collision> ()
  ((children         :exposed-field (<MFNode> "children" '()))
   (collide          :exposed-field (<SFBool> "collide" #t))
   (bbox-center      :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size        :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   (proxy            :field         (<SFNode> "proxy" #f))
   ;; event I/O
   (add-children     :event-in      (<MFNode> "addChildren"))
   (remove-children  :event-in      (<MFNode> "removeChildren"))
   (collide-time     :event-out     (<SFTime> "collideTime"))
   ))

(define-vrml-class <Color> ()
  ((color            :exposed-field (<MFColor> "color" '()))
   ))

(define-vrml-class <ColorInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFColor> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<SFColor> "value_changed"))
   ))

(define-vrml-class <Cone> ()
  ((bottom-radius    :field         (<SFFloat> "bottomRadius" 1.0))
   (height           :field         (<SFFloat> "height" 2.0))
   (side             :field         (<SFBool>  "side" #t))
   (bottom           :field         (<SFBool>  "bottom" #t))
   ))

(define-vrml-class <Coordinate> ()
  ((point            :exposed-field (<MFVec3f> "point" '()))
   ))

(define-vrml-class <CoordinateInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFVec3f> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<MFVec3f> "value_changed"))
   ))

(define-vrml-class <Cylinder> ()
  ((bottom           :field         (<SFBool> "bottom" #t))
   (height           :field         (<SFFloat> "height" 2))
   (radius           :field         (<SFFloat> "radius" 1))
   (side             :field         (<SFBool> "side" #t))
   (top              :field         (<SFBool> "top" #t))
   ))

(define-vrml-class <CylinderSensor> ()
  ((auto-offset      :exposed-field (<SFBool> "autoOffset" #t))
   (disk-angle       :exposed-field (<SFFloat> "diskAngle" 0.262))
   (enabled          :exposed-field (<SFBool> "enabled" #t))
   (max-angle        :exposed-field (<SFFloat> "maxAngle" -1))
   (min-angle        :exposed-field (<SFFloat> "minAngle" 0))
   (offset           :exposed-field (<SFFloat> "offset" 0))
   ;; event i/o
   (is-active        :event-out     (<SFBool> "isActive"))
   (rotation-changed :event-out     (<SFRotation> "rotation_changed"))
   (track-point-changed :event-out   (<SFVec3f> "trackPoint_changed"))
   ))

(define-vrml-class <DirectionalLight> ()
  ((ambient-intensity :exposed-field (<SFFloat> "ambientIntensity" 0))
   (color             :exposed-field (<SFColor> "color" '#(1 1 1)))
   (direction         :exposed-field (<SFVec3f> "direction" '#(0 0 -1)))
   (intensity         :exposed-field (<SFFloat> "intensity" 1))
   (on                :exposed-field (<SFBool> "on" #t))
   ))

(define-vrml-class <ElevationGrid> ()
  ((color             :exposed-field (<SFNode> "color" #f))
   (normal            :exposed-field (<SFNode> "normal" #f))
   (tex-coord         :exposed-field (<SFNode> "texCoord" #f))
   (height            :field         (<MFFloat> "height" '()))
   (ccw               :field         (<SFBool> "ccw" #t))
   (color-per-vertex  :field         (<SFBool> "colorPerVertex" #t))
   (crease-angle      :field         (<SFFloat> "creaseAngle" 0))
   (normal-per-vertex :field         (<SFBool> "normalPerVertex" #t))
   (solid             :field         (<SFBool> "solid" #t))
   (x-dimension       :field         (<SFInt32> "xDimension" 0))
   (x-spacing         :field         (<SFFloat> "xSpacing" 0))
   (z-dimension       :field         (<SFInt32> "zDimension" 0))
   (z-spacing         :field         (<SFFloat> "zSpacing" 0))
   ;; event i/o
   (set-height        :event-in      (<MFFloat> "set_height"))
   ))

(define-vrml-class <Extrusion> ()
  ((begin-cap         :field         (<SFBool> "beginCap" #t))
   (ccw               :field         (<SFBool> "ccw" #t))
   (convex            :field         (<SFBool> "convex" #t))
   (crease-angle      :field         (<SFFloat> "creaseAngle" 0))
   (cross-section     :field         (<MFVec2f> "crossSection"
                                                '(#(1 1) #(1 -1) #(-1 -1)
                                                  #(-1 1) #(1 1))))
   (endcap            :field         (<SFBool> "endCap" #t))
   (orientation       :field         (<MFRotation> "orientation"
                                                   '(#(0 0 1 0))))
   (scale             :field         (<MFVec2f> "scale" '(#(1 1))))
   (solid             :field         (<SFBool> "solid" #t))
   (spine             :field         (<MFVec3f> "spine" '(#(0 0 0) #(0 1 0))))
   ;; event i/o
   (set-cross-section :event-in      (<MFVec2f> "set_crossSection"))
   (set-orientation   :event-in      (<MFRotation> "set_orientation"))
   (set-scale         :event-in      (<MFVec2f> "set_scale"))
   (set-spine         :event-in      (<MFVec3f> "set_spine"))
   ))

(define-vrml-class <Fog> ()
  ((color             :exposed-field (<SFColor> "color" '#(1 1 1)))
   (fogtype           :exposed-field (<SFString> "fogType" "LINEAR"))
   (visibility-range  :exposed-field (<SFFloat> "visibilityRange" 0))
   ;; event i/o
   (set-bind          :event-in      (<SFBool> "set_bind"))
   (is-bound          :event-out     (<SFBool> "isBound"))
   ))

(define-vrml-class <FontStyle> ()
  ((family           :field         (<SFString> "family" "SERIF"))
   (horizontal       :field         (<SFBool> "horizontal" #t))
   (justify          :field         (<MFString> "justify" "BEGIN"))
   (language         :field         (<SFString> "language" ""))
   (left-to-right    :field         (<SFBool> "leftToRight" #t))
   (size             :field         (<SFFloat> "size" 1.0))
   (spacing          :field         (<SFFloat> "spacing" 1.0))
   (style            :field         (<SFString> "style" "PLAIN"))
   (top-to-bottom    :field         (<SFBool> "topToBottom" #t))
   ))

(define-vrml-class <Group> ()
  ((children         :exposed-field (<MFNode> "children" '()))
   (bbox-center      :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size        :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   ;; event I/O
   (add-children     :event-in      (<MFNode> "addChildren"))
   (remove-children  :event-in      (<MFNode> "removeChildren"))
   ))

(define-vrml-class <ImageTexture> ()
  ((url              :exposed-field (<MFString> "url" '()))
   (repeat-s         :field         (<SFBool> "repeatS" #t))
   (repeat-t         :field         (<SFBool> "repeatT" #t))
   ))

(define-vrml-class <IndexedFaceSet> ()
  ((color             :exposed-field (<SFNode> "color" #f))
   (coord             :exposed-field (<SFNode> "coord" #f))
   (normal            :exposed-field (<SFNode> "normal" #f))
   (tex-coord         :exposed-field (<SFNode> "texCoord" #f))
   (ccw               :field         (<SFBool> "ccw" #t))
   (color-index       :field         (<MFInt32> "colorIndex" '()))
   (color-per-vertex  :field         (<SFBool> "colorPerVertex" #t))
   (convex            :field         (<SFBool> "convex" #t))
   (coord-index       :field         (<MFInt32> "coordIndex" '()))
   (crease-angle      :field         (<SFFloat> "creaseAngle" 0))
   (normal-index      :field         (<MFInt32> "normalIndex" '()))
   (normal-per-vertex :field         (<SFBool> "normalPerVertex" #t))
   (solid             :field         (<SFBool> "solid" #t))
   (tex-coord-index   :field         (<MFInt32> "texCoordIndex" '()))
   ;; event i/o
   (set-color-index   :event-in      (<MFInt32> "set_colorIndex"))
   (set-coord-index   :event-in      (<MFInt32> "set_coordIndex"))
   (set-normal-index  :event-in      (<MFInt32> "set_normalIndex"))
   (set-texcoord-index :event-in     (<MFInt32> "set_texCoordIndex"))
   ))

(define-vrml-class <IndexedLineSet> ()
  ((color             :exposed-field (<SFNode> "color" #t))
   (coord             :exposed-field (<SFNode> "coord" #t))
   (color-index       :field         (<MFInt32> "colorIndex" '()))
   (color-per-vertex  :field         (<SFBool> "colorPerVertex" #t))
   (coord-index       :field         (<MFInt32> "coordIndex" '()))
   ;; event i/o
   (set-color-index   :event-in      (<MFInt32> "set_colorIndex"))
   (set-coord-index   :event-in      (<MFInt32> "set_coordIndex"))
   ))

(define-vrml-class <Inline> ()
  ((url              :exposed-field (<MFString> "url" '()))
   (bbox-center      :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size        :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   ))

(define-vrml-class <LOD> ()
  ((level            :exposed-field (<MFNode> "level" '()))
   (center           :field         (<SFVec3f> "center" '#(0 0 0)))
   (range            :field         (<MFFloat> "range" '()))
   ))

(define-vrml-class <Material> ()
  ((ambient-intensity :exposed-field (<SFFloat> "ambientIntensity" 0.2))
   (diffuse-color    :exposed-field (<SFColor> "diffuseColor" '#(0.8 0.8 0.8)))
   (emissive-color   :exposed-field (<SFColor> "emissiveColor" '#(0 0 0)))
   (shininess        :exposed-field (<SFFloat> "shininess" 0.2))
   (specular-color   :exposed-field (<SFColor> "specularColor" '#(0 0 0)))
   (transparency     :exposed-field (<SFFloat> "transparency" 0))
   ))

(define-vrml-class <MovieTexture> ()
  ((loop             :exposed-field (<SFBool> "loop" #f))
   (speed            :exposed-field (<SFFloat> "speed" 1))
   (start-time       :exposed-field (<SFTime> "startTime" 0))
   (stopt-ime        :exposed-field (<SFTime> "stopTime" 0))
   (url              :exposed-field (<MFString> "url" '()))
   (repeat-s         :field         (<SFBool> "repeatS" #t))
   (repeat-t         :field         (<SFBool> "repeatT" #t))
   ;; event i/o
   (duration-changed :event-out     (<SFFloat> "duration_changed"))
   (is-active        :event-out     (<SFBool> "isActive"))
   ))

(define-vrml-class <NavigationInfo> ()
  ((avatar-size      :exposed-field (<MFFloat> "avatarSize" '(0.25 1.6 0.75)))
   (head-light       :exposed-field (<SFBool> "headlight" #t))
   (speed            :exposed-field (<SFFloat> "speed" 1.0))
   (type             :exposed-field (<MFString> "type" '("WALK" "ANY")))
   (visibility-limit :exposed-field (<SFFloat> "visibilityLimit" 0))
   ;; event i/o
   (set-bind         :event-in      (<SFBool> "set_bind"))
   (is-bound         :event-out     (<SFBool> "isBonud"))
   ))

(define-vrml-class <Normal> ()
  ((vector           :exposed-field (<MFVec3f> "vector" '()))
   ))

(define-vrml-class <NormalInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFVec3f> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<MFVec3f> "value_changed"))
   ))

(define-vrml-class <OrientationInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFRotation> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<SFRotation> "value_changed"))
   ))

(define-vrml-class <PixelTexture> ()
  ((image            :exposed-field (<SFImage> "image" '#(0 0 0)))
   (repeat-s         :field         (<SFBool> "repeatS" #t))
   (repeat-t         :field         (<SFBool> "repeatT" #t))
   ))

(define-vrml-class <PlaneSensor> ()
  ((auto-offset      :exposed-field (<SFBool> "autoOffset" #t))
   (enabled          :exposed-field (<SFBool> "enabled" #t))
   (max-position     :exposed-field (<SFVec2f> "maxPosition" '#(-1 -1)))
   (min-position     :exposed-field (<SFVec2f> "minPosition" '#(0 0)))
   (offset           :exposed-field (<SFVec3f> "offset" '#(0 0 0)))
   ;; event i/o
   (is-active        :event-out     (<SFBool> "isActive"))
   (track-point-changed :event-out  (<SFVec3f> "trackPoint_changed"))
   (translation-changed :event-out  (<SFVec3f> "translation_changed"))
   ))

(define-vrml-class <PointLight> ()
  ((ambient-intensity :exposed-field (<SFFloat> "ambientIntensity" 0))
   (attenuation      :exposed-field (<SFVec3f> "attenuation" '#(1 0 0)))
   (color            :exposed-field (<SFColor> "color" '#(1 1 1)))
   (intensity        :exposed-field (<SFFloat> "intensity" 1))
   (location         :exposed-field (<SFVec3f> "location" '#(0 0 0)))
   (on               :exposed-field (<SFBool> "on" #t))
   (radius           :exposed-field (<SFFloat> "radius" 100))
   ))

(define-vrml-class <PointSet> ()
  ((color            :exposed-field (<SFNode> "color" #f))
   (coord            :exposed-field (<SFNode> "coord" #f))
   ))

(define-vrml-class <PositionInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFVec3f> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<SFVec3f> "value_changed"))
   ))

(define-vrml-class <ProximitySensor> ()
  ((center           :exposed-field (<SFVec3f> "center" '#(0 0 0)))
   (size             :exposed-field (<SFVec3f> "size" '#(0 0 0)))
   (enabled          :exposed-field (<SFBool> "enabled" #t))
   ;; event i/o
   (is-active        :event-out     (<SFBool> "isActive"))
   (position-changed :event-out     (<SFVec3f> "position_changed"))
   (orientation-changed :event-out  (<SFRotation> "orientation_changed"))
   (enter-time       :event-out     (<SFTime> "enterTime"))
   (exit-time        :event-out     (<SFTime> "exitTime"))
   ))

(define-vrml-class <ScalarInterpolator> ()
  ((key              :exposed-field (<MFFloat> "key" '()))
   (key-value        :exposed-field (<MFFloat> "keyValue" '()))
   ;; event i/o
   (set-fraction     :event-in      (<SFFloat> "set_fraction"))
   (value-changed    :event-out     (<SFFloat> "value_changed"))
   ))
  
(define-vrml-class <Shape> ()
  ((appearance        :exposed-field (<SFNode> "appearance" #f))
   (geometry          :exposed-field (<SFNode> "geometry" #f))
   ))

(define-vrml-class <Sound> ()
  ((direction         :exposed-field (<SFVec3f> "direction" '#(0 0 1)))
   (intensity         :exposed-field (<SFFloat> "intensity" 1))
   (location          :exposed-field (<SFVec3f> "location" '#(0 0 0)))
   (max-back          :exposed-field (<SFFloat> "maxBack" 10))
   (max-front         :exposed-field (<SFFloat> "maxFront" 10))
   (min-back          :exposed-field (<SFFloat> "minBack" 1))
   (min-front         :exposed-field (<SFFloat> "minFront" 1))
   (priority          :exposed-field (<SFFloat> "priority" 0))
   (source            :exposed-field (<SFNode> "source" #f))
   (spatialize        :field         (<SFBool> "spatialize" #t))
   ))

(define-vrml-class <Sphere> ()
  ((radius            :field         (<SFFloat> "radius" 1))
   ))

(define-vrml-class <SphereSensor> ()
  ((auto-offset       :exposed-field (<SFBool> "autoOffset" #t))
   (enabled           :exposed-field (<SFBool> "enabled" #t))
   (offset            :exposed-field (<SFRotation> "offset" '#(0 1 0 0)))
   ;; event i/o
   (is-active         :event-out     (<SFBool> "isActive"))
   (rotation-changed  :event-out     (<SFRotation> "rotation_changed"))
   (track-point-changed :event-out    (<SFVec3f> "trackPoint_changed"))
   ))

(define-vrml-class <SpotLight> ()
  ((ambient-intensity :exposed-field (<SFFloat> "ambientIntensity" 0))
   (attenuation       :exposed-field (<SFVec3f> "attenuation" '#(1 0 0)))
   (beam-width        :exposed-field (<SFFloat> "beamWidth" 1.570796))
   (color             :exposed-field (<SFColor> "color" '#(1 1 1)))
   (cutoff-angle      :exposed-field (<SFFloat> "curOffAngle" 0.785398))
   (direction         :exposed-field (<SFVec3f> "direction" '#(0 0 -1)))
   (intensity         :exposed-field (<SFFloat> "intensity" 1))
   (location          :exposed-field (<SFVec3f> "location" '#(0 0 0)))
   (on                :exposed-field (<SFBool> "on" #t))
   (radius            :exposed-field (<SFFloat> "radius" 100))
   ))

(define-vrml-class <Switch> ()
  ((choice            :exposed-field (<MFNode> "choice" '()))
   (which-choice      :exposed-field (<SFInt32> "whichChoice" -1))
   ))

(define-vrml-class <Text> ()
  ((string            :exposed-field (<MFString> "string" '()))
   (font-style        :exposed-field (<SFNode> "fontStyle" #f))
   (length            :exposed-field (<MFFloat> "length" '()))
   (max-extent        :exposed-field (<SFFloat> "maxExtent" 0.0))
   ))

(define-vrml-class <TextureCoordinate> ()
  ((point             :exposed-field (<MFVec2f> "point" '()))
   ))

(define-vrml-class <TextureTransform> ()
  ((center            :exposed-field (<SFVec2f> "center" '#(0 0)))
   (rotation          :exposed-field (<SFFloat> "rotation" 0))
   (scale             :exposed-field (<SFVec2f> "scale" '#(1 1)))
   (translation       :exposed-field (<SFVec2f> "translation" '#(0 0)))
   ))

(define-vrml-class <TimeSensor> ()
  ((cycle-interval    :exposed-field (<SFTime> "cycleInterval" 1))
   (enabled           :exposed-field (<SFBool> "enabled" #t))
   (loop              :exposed-field (<SFBool> "loop" #f))
   (start-time        :exposed-field (<SFTime> "startTime" 0))
   (stop-time         :exposed-field (<SFTime> "stopTime" 0))
   ;; event i/o
   (cycle-time        :event-out     (<SFTime> "cycleTime"))
   (fraction-changed  :event-out     (<SFFloat> "fraction_changed"))
   (is-active         :event-out     (<SFBool> "isActive"))
   (time              :event-out     (<SFTime> "time"))
   ))

(define-vrml-class <TouchSensor> ()
  ((enabled             :exposed-field (<SFBool> "enabled" #t))
   ;; event i/o
   (hit-normal-changed  :event-out   (<SFVec3f> "hitNormal_changed"))
   (hit-point-changed   :event-out   (<SFVec3f> "hitPoint_changed"))
   (hit-tex-coord-changed :event-out   (<SFVec3f> "hitTexCoord_changed"))
   (is-active           :event-out   (<SFBool> "isActive"))
   (is-over             :event-out   (<SFBool> "isOver"))
   (touch-time          :event-out   (<SFTime> "touchTime"))
   ))

(define-vrml-class <Transform> ()
  ((center            :exposed-field (<SFVec3f> "center" '#(0 0 0)))
   (children          :exposed-field (<MFNode> "children" '()))
   (rotation          :exposed-field (<SFRotation> "rotation" '#(0 0 1 0)))
   (scale             :exposed-field (<SFVec3f> "scale" '#(1 1 1)))
   (scale-orientation :exposed-field (<SFRotation> "scaleOrientation"
                                                  '#(0 0 1 0)))
   (translation       :exposed-field (<SFVec3f> "translation" '#(0 0 0)))
   (bbox-center       :field         (<SFVec3f> "bboxCenter" '#(0 0 0)))
   (bbox-size         :field         (<SFVec3f> "bboxSize" '#(-1 -1 -1)))
   ;; event i/o
   (add-children      :event-in      (<MFNode> "addChildren"))
   (remove-children   :event-in      (<MFNode> "removeChildren"))
   ))

(define-vrml-class <Viewpoint> ()
  ((field-of-view     :exposed-field (<SFFloat> "fieldOfView" 0.785398))
   (jump              :exposed-field (<SFBool> "jump" #t))
   (orientation       :exposed-field (<SFRotation> "orientation" '#(0 0 1 0)))
   (position          :exposed-field (<SFVec3f> "position" '#(0 0 10)))
   (description       :field         (<SFString> "description" ""))
   ;; event i/o
   (set-bind          :event-in      (<SFBool> "set_bind"))
   (bind-time         :event-out     (<SFTime> "bindTime"))
   (is-bound          :event-out     (<SFBool> "isBound"))
   ))

(define-vrml-class <VisibilitySensor> ()
  ((center            :exposed-field (<SFVec3f> "center" '#(0 0 0)))
   (enabled           :exposed-field (<SFBool> "enabled" #t))
   (size              :exposed-field (<SFVec3f> "size" '#(0 0 0)))
   ;; event i/o
   (enter-time        :event-out     (<SFTime> "enterTime"))
   (exit-time         :event-out     (<SFTime> "exitTime"))
   (is-active         :event-out     (<SFBool> "isActive"))
   ))

(define-vrml-class <WorldInfo> ()
  ((info              :field         (<MFString> "info" '()))
   (title             :field         (<SFString> "title" ""))
   ))

;; TODO: script node

(provide "x3d/vrml")
