;;;
;;; banyan.server.cvs - cvs interface
;;;
;;;  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: cvs.scm,v 1.4 2003/02/21 10:32:20 shirok Exp $
;;;

;; this file is autoloaded to banyan.server module

(define-module banyan.server.cvs
  (use gauche.process)
  (use srfi-1)
  (use srfi-2)
  (use srfi-13)
  (use file.util)
  (extend banyan.server)
  (export cvs-commit cvs-add cvs-add-project cvs-add-path*)
  )
(select-module banyan.server.cvs)

;;------------------------------------------------------------
;; CVS Wrappers
;;
(define (cvs-command repo . args)
  (let ((p (apply run-process "cvs"
                  "-d" (build-path (ref repo 'root) "cvsroot") "-q"
                  :wait #t
                  :input "/dev/null" :output "/dev/null" :error "/dev/null"
                  args)))
    (zero? (process-exit-status p))))

(define (cvs-commit repo new? path)
  (when new?
    (or (cvs-command repo "add" (repo-path repo path))
        (error "cvs add failed" path)))
  (or (cvs-command repo "commit" "-m" "checkin" (repo-path repo path))
      (error "cvs commit failed" path)))

(define (cvs-add-dir repo path)
  (let ((full (repo-path repo path)))
    (ensure-directory full)
    (unless (file-exists? (build-path full "CVS"))
      (sys-chdir (repo-path repo))
      (or (cvs-command repo "add" path)
          (error "cvs add failed" path)))))

(define (cvs-add-project repo name)
  (let ((path (repo-path repo name)))
    (ensure-directory path)
    (unless (file-exists? (build-path path "CVS"))
      (let ((old (sys-getcwd)))
        (sys-chdir path)
        (or (cvs-command repo "import"
                         "-m" #`"new project by ,(current-user)"
                         name "banyan" "new_project")
            (error "cvs import failed" path))
        (sys-chdir (repo-path repo))
        (sys-rmdir name)
        (or (cvs-command repo "co" name)
            (error "cvs checkout failed" path))
        ))))

(define (cvs-add-path* repo path)
  (pair-fold-right
   (lambda (comps knil)
     (if (null? (cdr comps))
         (cvs-add-project repo (car comps))
         (cvs-add-dir repo (string-join (reverse comps) "/"))))
   #f
   (reverse (string-split path #\/))))

;;------------------------------------------------------------
;; Filesystem utilities
;;
(define (is-cvs-directory? path)
  (and (file-exists? path)
       (file-is-directory? path)
       (file-exists? (build-path path "CVS"))))

(define (ensure-directory path)
  (unless (file-exists? path) (make-directory* path))
  (unless (file-is-directory? path)
    (errorf "regular file ~s gets in my way" path)))


(provide "banyan/server/cvs")
