;;;
;;; banyan.server.data - data I/O handling
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: data.scm,v 1.1 2003/02/21 10:32:21 shirok Exp $
;;;

;; Handling data body I/O.

(define-module banyan.server.data
  (extend banyan.server)
  (export repository-read repository-write)
  )
(select-module banyan.server.data)

;;================================================
;; Reading/writing
;;

;; Returns list of metadata and data body
(define (repository-read repo path)
  (let ((file (repo-path repo path part-file)))
    (call-with-input-file file port->sexp-list)))

;;================================================
;; Dummy classes to deal with srfi-10 syntax
;;

(define-syntax define-dummy-class
  (syntax-rules ()
    ((_ name tag)
     (begin
       (define-class name () ((data :init-keyword :data)))
       (define-method write-object ((self name) port)
         (format port "#,~s" (cons 'tag (ref self 'data))))
       (define-reader-ctor 'tag
         (lambda args (make name :data args)))
       ))))

(define-dummy-class <dummy-vector4f> vector4f)
(define-dummy-class <dummy-point4f>  point4f)
(define-dummy-class <dummy-matrix4f> matrix4f)
(define-dummy-class <dummy-quatf>    quatf)
(define-dummy-class <dummy-vector4f-array> vector4f-array)
(define-dummy-class <dummy-point4f-array> point4f-array)
(define-dummy-class <dummy-node> node)
(define-dummy-class <dummy-node-ref> node-ref)
(define-dummy-class <dummy-node-ext-ref> node-ext-ref)
(define-dummy-class <dummy-part> part)

(provide "banyan/server/data")
