;;;
;;; banyan.server.export - exporter handling
;;;
;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
;;;
;;;  Permission to use, copy, modify, distribute this software and
;;;  accompanying documentation for any purpose is hereby granted,
;;;  provided that existing copyright notices are retained in all
;;;  copies and that this notice is included verbatim in all
;;;  distributions.
;;;  This software is provided as is, without express or implied
;;;  warranty.  In no circumstances the author(s) shall be liable
;;;  for any damages arising out of the use of this software.
;;;
;;;  $Id: export.scm,v 1.3 2003/02/22 04:11:32 shirok Exp $
;;;

;; Currently only VRML is supported.

(define-module banyan.server.export
  (use file.util)
  (extend banyan.server)
  (export server-export-to-vrml)
  )
(select-module banyan.server.export)

(define (server-export-to-vrml repo path)
  (let ((part (repo-path repo path part-file))
        (wrl  (repo-path repo path #`",(sys-basename path).wrl"))
        )
    (unless (and (file-exists? wrl)
                 (file-mtime>=? wrl part))
      (generate-vrml repo path wrl))
    (send-wrl wrl)))

(define (send-wrl wrl-file)
  (print "Content-type: application/x-vrml")
  (print)
  (call-with-input-file wrl-file
    (lambda (in)
      (copy-port in (current-output-port))))
  )

(define (generate-vrml repo path wrl)
  (let1 exporter (ref repo 'vrml-exporter)
    (unless (and exporter (file-is-executable? exporter))
      (error "I don't have a VRML exporter"))
    (receive (p tmpfile) (sys-mkstemp wrl)
      (close-output-port p)
      (let1 cmd #`",exporter -o ,tmpfile -d ,(ref repo 'root)/checkout ,(sanitize-path path)"
        (plog "executing ~a" cmd)
        (let1 r (sys-system cmd)
          (if (= r 0)
              (sys-rename tmpfile wrl)
              (begin (sys-unlink tmpfile)
                     (error "couldn't generate VRML file" wrl))))
        ))
    ))

(provide "banyan/server/export")

