/*
 * banyan.c - banyan common library
 *
 *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
 *
 *  Permission to use, copy, modify, distribute this software and
 *  accompanying documentation for any purpose is hereby granted,
 *  provided that existing copyright notices are retained in all
 *  copies and that this notice is included verbatim in all
 *  distributions.
 *  This software is provided as is, without express or implied
 *  warranty.  In no circumstances the author(s) shall be liable
 *  for any damages arising out of the use of this software.
 *
 *  $Id: banyan.c,v 1.7 2003/01/01 01:04:01 shirok Exp $
 */

#include <gauche/extend.h>
#include "banyan.h"

/*==============================================================
 * BanyanNode
 */

SCM_DEFINE_ABSTRACT_CLASS(Banyan_NodeClass, NULL);

ScmClass *Banyan_NodeCPL[] = {
    SCM_CLASS_STATIC_PTR(Banyan_NodeClass),
    SCM_CLASS_STATIC_PTR(Scm_TopClass),
    NULL
};

ScmObj Banyan_NodeGetNode(ScmObj node)
{
    return BANYAN_NODE(node)->node;
}

void Banyan_NodeSetNode(ScmObj node, ScmObj value)
{
    ScmObj nodeclass = Banyan_GetNodeClass();
    if (!SCM_ISA(value, SCM_CLASS(nodeclass))) {
        Scm_Error("<node> required, but got %S", value);
    }
    BANYAN_NODE(node)->node = value;
}

/*==============================================================
 * Utilities
 */

/* Returns class ptr of Scheme-defined class <node>.
   This should be called after Scheme libs are loaded. */
ScmObj Banyan_GetNodeClass(void)
{
    static ScmGloc *nodeloc = NULL;
    if (nodeloc) {
        return SCM_GLOC_GET(nodeloc);
    }
    nodeloc = Scm_FindBinding(SCM_MODULE(SCM_FIND_MODULE("banyan", FALSE)),
                              SCM_SYMBOL(SCM_INTERN("<node>")),
                              FALSE);
    if (nodeloc == NULL) {
        Scm_Error("internal error: Banyan_NodeClass is called before loading banyan Scheme library");
    }
    return SCM_GLOC_GET(nodeloc);
}

static ScmClassStaticSlotSpec node_slots[] = {
    SCM_CLASS_SLOT_SPEC("node", Banyan_NodeGetNode, Banyan_NodeSetNode),
    { NULL }
};

/*==============================================================
 * Initialization
 */

extern void Banyan__InitShape(ScmModule*);
extern void Banyan__InitTransform(ScmModule*);
extern void Banyan__InitLight(ScmModule*);
extern void Scm_Init_bnlib(ScmModule*);

ScmObj Banyan_RenderNoshade   = SCM_FALSE;
ScmObj Banyan_RenderShade     = SCM_FALSE;
ScmObj Banyan_RenderSelection = SCM_FALSE;

void Scm_Init_banyan(void)
{
    ScmModule *mod;
    SCM_INIT_EXTENSION(banyan);
    mod = SCM_MODULE(SCM_FIND_MODULE("banyan", TRUE));

    Banyan_RenderNoshade   = SCM_INTERN("noshade");
    Banyan_RenderShade     = SCM_INTERN("shade");
    Banyan_RenderSelection = SCM_INTERN("selection");

    Scm_InitBuiltinClass(&Banyan_NodeClass, "<%node>", node_slots,
                         sizeof(BanyanNode), mod);
    
    Banyan__InitShape(mod);
    Banyan__InitTransform(mod);
    Banyan__InitLight(mod);
    Scm_Init_bnlib(mod);
}
