/*
 * banyan.h - libbanyan main header
 *
 *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
 *
 *  Permission to use, copy, modify, distribute this software and
 *  accompanying documentation for any purpose is hereby granted,
 *  provided that existing copyright notices are retained in all
 *  copies and that this notice is included verbatim in all
 *  distributions.
 *  This software is provided as is, without express or implied
 *  warranty.  In no circumstances the author(s) shall be liable
 *  for any damages arising out of the use of this software.
 *
 *  $Id: banyan.h,v 1.10 2003/12/10 01:00:50 shirok Exp $
 */

#ifndef BANYAN_H
#define BANYAN_H

#include <GL/gl.h>
#include <GL/glu.h>
#include <gauche.h>
#include <gauche/class.h>
#include <gauche/uvector.h>
#include <gauche/math3d.h>

/*
 * The C-counterpart of Scheme <node>.
 *
 *  The current version of Gauche has a limitation in subclassing
 *  between C-defined classes and Scheme-defined classes.  Basically
 *  you can't define a C class that inherits a Scheme class, nor
 *  define a C class whose metaclass is a Scheme class.
 *  This limitation will be removed in later version of Gauche, but
 *  for the time being, I'll define two class trees---one for Scheme
 *  and the other for C. 
 *  The 'bridge' code will be encapsulated in banyan/meta.scm.
 */

typedef struct BanyanNodeRec {
    SCM_HEADER;
    ScmObj node;                /* back ptr to the Scheme node. */
} BanyanNode;

#define BANYAN_HEADER   SCM_HEADER; ScmObj node
#define BANYAN_HEADER_INIT(obj)   (BANYAN_NODE(obj)->node = SCM_FALSE)
#define BANYAN_NODE_SLOT_SPEC \
  SCM_CLASS_SLOT_SPEC("node", Banyan_NodeGetNode, Banyan_NodeSetNode)

extern ScmClass *Banyan_NodeCPL[];
SCM_CLASS_DECL(Banyan_NodeClass);
#define BANYAN_NODE_CLASS   (&Banyan_NodeClass)
#define BANYAN_NODE(obj)    ((BanyanNode*)(obj))
#define BANYAN_NODE_P(obj)  SCM_ISA(obj, BANYAN_NODE_CLASS)

extern ScmObj Banyan_GetNodeClass(void);

extern ScmObj Banyan_NodeGetNode(ScmObj node);
extern void   Banyan_NodeSetNode(ScmObj node, ScmObj n);

#include "shape.h"
#include "transform.h"
#include "light.h"

/*
 * Common stuff
 */

/* frequently used symbols */
extern ScmObj Banyan_RenderNoshade;
extern ScmObj Banyan_RenderShade;
extern ScmObj Banyan_RenderSelection;

#define BANYAN_RENDER_NOSHADE   Banyan_RenderNoshade
#define BANYAN_RENDER_SHADE     Banyan_RenderShade
#define BANYAN_RENDER_SELECTION Banyan_RenderSelection

#endif /* BANYAN_H */
