/*
 * light.h - light handling
 *
 *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
 *
 *  Permission to use, copy, modify, distribute this software and
 *  accompanying documentation for any purpose is hereby granted,
 *  provided that existing copyright notices are retained in all
 *  copies and that this notice is included verbatim in all
 *  distributions.
 *  This software is provided as is, without express or implied
 *  warranty.  In no circumstances the author(s) shall be liable
 *  for any damages arising out of the use of this software.
 *
 *  $Id: light.h,v 1.4 2003/01/01 01:04:01 shirok Exp $
 */

#ifndef BANYAN_LIGHT_H
#define BANYAN_LIGHT_H

/*
 * Internal light structure.
 *
 *  Light class is defined in C because C-defined shader need to
 *  look up the light.
 */
typedef struct BanyanLightRec BanyanLight;
typedef void (*BanyanLightRenderer)(BanyanLight *, int);

struct BanyanLightRec {
    BANYAN_HEADER;
    BanyanLightRenderer renderer; /* render method */
    /* Public */
    ScmVector4f *ambient;       /* ambient color */
    ScmVector4f *diffuse;       /* diffuse color */
    ScmVector4f *specular;      /* specular color */
    BanyanTransform *transform; /* global matrix for position & direction */
    float spot_exponent;
    float spot_cutoff;
    float constant_attenuation;
    float linear_attenuation;
    float quadratic_attenuation;
};

SCM_CLASS_DECL(Banyan_LightClass);
#define BANYAN_LIGHT_CLASS   (&Banyan_LightClass)
#define BANYAN_LIGHT(obj)    ((BanyanLight*)(obj))
#define BANYAN_LIGHT_P(obj)  SCM_ISA(obj, BANYAN_LIGHT_CLASS)

extern ScmObj Banyan_MakeLight(void);

/*
 * Lightgroup
 *
 *   Lightgroup keeps a set of lights that are active at the same time.
 */

#define BANYAN_MAX_HARDWARE_LIGHTS 8

typedef struct BanyanLightGroupRec {
    BANYAN_HEADER;
    /* Public */
    ScmVector *lights;          /* vector of lights */
} BanyanLightGroup;

SCM_CLASS_DECL(Banyan_LightGroupClass);
#define BANYAN_LIGHT_GROUP_CLASS   (&Banyan_LightGroupClass)
#define BANYAN_LIGHT_GROUP(obj)    ((BanyanLightGroup*)(obj))
#define BANYAN_LIGHT_GROUP_P(obj)  SCM_ISA(obj, BANYAN_LIGHT_GROUP_CLASS)

extern ScmObj Banyan_MakeLightGroup(void);
extern void Banyan_LightGroupRender(BanyanLightGroup *lg, ScmObj mode);

#endif /* BANYAN_LIGHT_H */
