/*
 * shape.h - shape handling
 *
 *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
 *
 *  Permission to use, copy, modify, distribute this software and
 *  accompanying documentation for any purpose is hereby granted,
 *  provided that existing copyright notices are retained in all
 *  copies and that this notice is included verbatim in all
 *  distributions.
 *  This software is provided as is, without express or implied
 *  warranty.  In no circumstances the author(s) shall be liable
 *  for any damages arising out of the use of this software.
 *
 *  $Id: shape.h,v 1.7 2003/01/01 01:04:01 shirok Exp $
 */

#ifndef BANYAN_SHAPE_H
#define BANYAN_SHAPE_H

/*
 * Internal shape structure.
 *
 *   Shape is kind of optimized for displaying, instead of editing.
 *   The main data is kept in the way that it can be rendered efficiently
 *   with GL's vertex array.
 *
 *   'Faces' and 'edges' are not really a first-class citizen.  They can
 *   only be accessed via iteator.
 *
 */
typedef struct BanyanShapeRec {
    BANYAN_HEADER;
    /* Public */
    ScmObj vertices;            /* ScmPoint4fArray */
    ScmObj normals;             /* ScmVector4fArray, for now */
    ScmObj faces;               /* List of (gl_type index_vector) */
    int numFaces;               /* R/O: # of faces derived from faces */
    ScmObj facetNormals;        /* R/O: ScmVector4fArray */
} BanyanShape;

SCM_CLASS_DECL(Banyan_ShapeClass);
#define BANYAN_SHAPE_CLASS   (&Banyan_ShapeClass)
#define BANYAN_SHAPE(obj)    ((BanyanShape*)(obj))
#define BANYAN_SHAPE_P(obj)  SCM_ISA(obj, BANYAN_SHAPE_CLASS)

extern ScmObj Banyan_MakeShape(void);
extern void   Banyan_ShapeRender(BanyanShape *shape, ScmObj mode);

extern void   Banyan_ShapeSetFacetNormals(BanyanShape *shape);
extern void   Banyan_ShapeSetVertexNormals(BanyanShape *shape);

extern void   Banyan_ShapeRenderNormals(double scale);

/*
 * Face iterator
 */
typedef struct BanyanFaceIterRec {
    BanyanShape *shape;
    ScmObj facep;               /* ptr to the face list */
    int mode;                   /* current mode */
    ScmUVector *ivec;           /* current index vector */
    int index;                  /* current index into the index vector */
    int size;                   /* size of current index vector */
    int count;                  /* face count */
} BanyanFaceIter;

#define BANYAN_FACE_ITER_END(iter)   (SCM_NULLP((iter)->facep))
#define BANYAN_FACE_ITER_COUNT(iter) ((iter)->count)
extern int    Banyan_FaceIterInit(BanyanShape *shape, BanyanFaceIter *iter);
extern int    Banyan_FaceIterNext(BanyanFaceIter *iter);
extern int    Banyan_FaceIterGetNumVertices(BanyanFaceIter *iter);
extern void   Banyan_FaceIterGetIndices(BanyanFaceIter *iter,
                                        GLuint indices[]);
extern int    Banyan_FaceIterGetIndex(BanyanFaceIter *iter, int vindex);
extern float *Banyan_FaceIterGetVertexPosition(BanyanFaceIter *iter,
                                               int vindex);
extern float *Banyan_FaceIterGetVertexNormal(BanyanFaceIter *iter,
                                             int vindex);
extern void   Banyan_FaceIterGetCenter(BanyanFaceIter *iter, float pt[]);

extern ScmObj Banyan_VMFaceFoldIndices(BanyanShape *shape,
                                       ScmObj proc,
                                       ScmObj knil);
extern ScmObj Banyan_VMFaceFoldCountIndices(BanyanShape *shape,
                                            ScmObj proc,
                                            ScmObj knil);
extern ScmObj Banyan_VMFaceFoldVertices(BanyanShape *shape,
                                        ScmObj proc,
                                        ScmObj knil);
extern ScmObj Banyan_VMFaceFoldCountVertices(BanyanShape *shape,
                                             ScmObj proc,
                                             ScmObj knil);
extern ScmObj Banyan_VMFaceFoldCenter(BanyanShape *shape,
                                      ScmObj proc,
                                      ScmObj knil);

#endif /* BANYAN_SHAPE_H */
