/*
 * transform.h - transform handling
 *
 *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
 *
 *  Permission to use, copy, modify, distribute this software and
 *  accompanying documentation for any purpose is hereby granted,
 *  provided that existing copyright notices are retained in all
 *  copies and that this notice is included verbatim in all
 *  distributions.
 *  This software is provided as is, without express or implied
 *  warranty.  In no circumstances the author(s) shall be liable
 *  for any damages arising out of the use of this software.
 *
 *  $Id: transform.h,v 1.4 2003/02/23 02:41:58 shirok Exp $
 */

#ifndef BANYAN_TRANSFORM_H
#define BANYAN_TRANSFORM_H

/*
 * Internal transform structure.
 *
 *   NB: Scheme <transform> object points C transform object.
 *     C's transform->node points back to Scheme <transform> object.
 *     This double-link is maintained by Scheme-side.
 */
typedef struct BanyanTransformRec {
    BANYAN_HEADER;
    /* Public */
    ScmMatrix4f *gmatrix;         /* global matrix */
    ScmMatrix4f *lmatrix;         /* local  matrix */
    ScmMatrix4f *cmatrix;         /* cancel matrix */
    ScmMatrix4f *imatrix;         /* inverse matrix */
    struct BanyanTransformRec *parent; /* parent xform */
} BanyanTransform;

SCM_CLASS_DECL(Banyan_TransformClass);
#define BANYAN_TRANSFORM_CLASS   (&Banyan_TransformClass)
#define BANYAN_TRANSFORM(obj)    ((BanyanTransform*)(obj))
#define BANYAN_TRANSFORM_P(obj)  SCM_ISA(obj, BANYAN_TRANSFORM_CLASS)

extern ScmObj Banyan_MakeTransform(void);
extern void   Banyan_TransformUpdate(BanyanTransform *transform);

#endif /* BANYAN_TRANSFORM_H */
